# -*- coding: iso-8859-1 -*-
# vim: set ft=python ts=4 sw=4 expandtab:
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
#              C E D A R
#          S O L U T I O N S       "Software done right."
#           S O F T W A R E
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
# Copyright (c) 2007,2010,2015 Kenneth J. Pronovici.
# All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License,
# Version 2, as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# Copies of the GNU General Public License are available from
# the Free Software Foundation website, http://www.gnu.org/.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
# Author   : Kenneth J. Pronovici <pronovic@ieee.org>
# Language : Python 3 (>= 3.7)
# Project  : Cedar Backup, release 3
# Purpose  : Tests encrypt extension functionality.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

########################################################################
# Module documentation
########################################################################

"""
Unit tests for CedarBackup3/extend/encrypt.py.

Code Coverage
=============

   This module contains individual tests for the the public classes implemented
   in extend/encrypt.py.  There are also tests for some of the private
   functions.

Naming Conventions
==================

   I prefer to avoid large unit tests which validate more than one piece of
   functionality, and I prefer to avoid using overly descriptive (read: long)
   test names, as well.  Instead, I use lots of very small tests that each
   validate one specific thing.  These small tests are then named with an index
   number, yielding something like ``testAddDir_001`` or ``testValidate_010``.
   Each method has a docstring describing what it's supposed to accomplish.  I
   feel that this makes it easier to judge how important a given failure is,
   and also makes it somewhat easier to diagnose and fix individual problems.

Testing XML Extraction
======================

   It's difficult to validated that generated XML is exactly "right",
   especially when dealing with pretty-printed XML.  We can't just provide a
   constant string and say "the result must match this".  Instead, what we do
   is extract a node, build some XML from it, and then feed that XML back into
   another object's constructor.  If that parse process succeeds and the old
   object is equal to the new object, we assume that the extract was
   successful.

   It would arguably be better if we could do a completely independent check -
   but implementing that check would be equivalent to re-implementing all of
   the existing functionality that we're validating here!  After all, the most
   important thing is that data can move seamlessly from object to XML document
   and back to object.

Full vs. Reduced Tests
======================

   Some Cedar Backup regression tests require a specialized environment in
   order to run successfully.  This environment won't necessarily be available
   on every build system out there (for instance, on a Debian autobuilder).
   Because of this, the default behavior is to run a "reduced feature set" test
   suite that has no surprising system, kernel or network requirements.  If you
   want to run all of the tests, set ENCRYPTTESTS_FULL to "Y" in the environment.

   In this module, the primary dependency is that for some tests, GPG must have
   access to the public key EFD75934.  There is also an assumption that GPG
   does *not* have access to a public key for anyone named "Bogus J. User" (so
   we can test failure scenarios).

@author Kenneth J. Pronovici <pronovic@ieee.org>
"""


########################################################################
# Import modules and do runtime validations
########################################################################

import os
import tempfile
import unittest

from CedarBackup3.extend.encrypt import EncryptConfig, LocalConfig, _encryptDailyDir, _encryptFile, _encryptFileWithGpg
from CedarBackup3.filesystem import FilesystemList
from CedarBackup3.testutil import buildPath, configureLogging, extractTar, failUnlessAssignRaises, findResources, removedir
from CedarBackup3.xmlutil import createOutputDom, serializeDom

#######################################################################
# Module-wide configuration and constants
#######################################################################

DATA_DIRS = [
    "./data",
    "./tests/data",
]
RESOURCES = [
    "encrypt.conf.1",
    "encrypt.conf.2",
    "tree1.tar.gz",
    "tree2.tar.gz",
    "tree8.tar.gz",
    "tree15.tar.gz",
    "tree16.tar.gz",
    "tree17.tar.gz",
    "tree18.tar.gz",
    "tree19.tar.gz",
    "tree20.tar.gz",
]

VALID_GPG_RECIPIENT = "EFD75934"
INVALID_GPG_RECIPIENT = "Bogus J. User"

INVALID_PATH = "bogus"  # This path name should never exist


#######################################################################
# Utility functions
#######################################################################


def runAllTests():
    """Returns true/false depending on whether the full test suite should be run."""
    if "ENCRYPTTESTS_FULL" in os.environ:
        return os.environ["ENCRYPTTESTS_FULL"] == "Y"
    else:
        return False


#######################################################################
# Test Case Classes
#######################################################################

##########################
# TestEncryptConfig class
##########################


class TestEncryptConfig(unittest.TestCase):

    """Tests for the EncryptConfig class."""

    ################
    # Setup methods
    ################

    @classmethod
    def setUpClass(cls):
        configureLogging()

    ##################
    # Utility methods
    ##################

    def failUnlessAssignRaises(self, exception, obj, prop, value):
        """Equivalent of :any:`failUnlessRaises`, but used for property assignments instead."""
        failUnlessAssignRaises(self, exception, obj, prop, value)

    ############################
    # Test __repr__ and __str__
    ############################

    def testStringFuncs_001(self):
        """
      Just make sure that the string functions don't have errors (i.e. bad variable names).
      """
        obj = EncryptConfig()
        obj.__repr__()
        obj.__str__()

    ##################################
    # Test constructor and attributes
    ##################################

    def testConstructor_001(self):
        """
      Test constructor with no values filled in.
      """
        encrypt = EncryptConfig()
        self.assertEqual(None, encrypt.encryptMode)
        self.assertEqual(None, encrypt.encryptTarget)

    def testConstructor_002(self):
        """
      Test constructor with all values filled in, with valid values.
      """
        encrypt = EncryptConfig("gpg", "Backup User")
        self.assertEqual("gpg", encrypt.encryptMode)
        self.assertEqual("Backup User", encrypt.encryptTarget)

    def testConstructor_003(self):
        """
      Test assignment of encryptMode attribute, None value.
      """
        encrypt = EncryptConfig(encryptMode="gpg")
        self.assertEqual("gpg", encrypt.encryptMode)
        encrypt.encryptMode = None
        self.assertEqual(None, encrypt.encryptMode)

    def testConstructor_004(self):
        """
      Test assignment of encryptMode attribute, valid value.
      """
        encrypt = EncryptConfig()
        self.assertEqual(None, encrypt.encryptMode)
        encrypt.encryptMode = "gpg"
        self.assertEqual("gpg", encrypt.encryptMode)

    def testConstructor_005(self):
        """
      Test assignment of encryptMode attribute, invalid value (empty).
      """
        encrypt = EncryptConfig()
        self.assertEqual(None, encrypt.encryptMode)
        self.failUnlessAssignRaises(ValueError, encrypt, "encryptMode", "")
        self.assertEqual(None, encrypt.encryptMode)

    def testConstructor_006(self):
        """
      Test assignment of encryptTarget attribute, None value.
      """
        encrypt = EncryptConfig(encryptTarget="Backup User")
        self.assertEqual("Backup User", encrypt.encryptTarget)
        encrypt.encryptTarget = None
        self.assertEqual(None, encrypt.encryptTarget)

    def testConstructor_007(self):
        """
      Test assignment of encryptTarget attribute, valid value.
      """
        encrypt = EncryptConfig()
        self.assertEqual(None, encrypt.encryptTarget)
        encrypt.encryptTarget = "Backup User"
        self.assertEqual("Backup User", encrypt.encryptTarget)

    def testConstructor_008(self):
        """
      Test assignment of encryptTarget attribute, invalid value (empty).
      """
        encrypt = EncryptConfig()
        self.assertEqual(None, encrypt.encryptTarget)
        self.failUnlessAssignRaises(ValueError, encrypt, "encryptTarget", "")
        self.assertEqual(None, encrypt.encryptTarget)

    ############################
    # Test comparison operators
    ############################

    def testComparison_001(self):
        """
      Test comparison of two identical objects, all attributes None.
      """
        encrypt1 = EncryptConfig()
        encrypt2 = EncryptConfig()
        self.assertEqual(encrypt1, encrypt2)
        self.assertTrue(encrypt1 == encrypt2)
        self.assertTrue(not encrypt1 < encrypt2)
        self.assertTrue(encrypt1 <= encrypt2)
        self.assertTrue(not encrypt1 > encrypt2)
        self.assertTrue(encrypt1 >= encrypt2)
        self.assertTrue(not encrypt1 != encrypt2)

    def testComparison_002(self):
        """
      Test comparison of two identical objects, all attributes non-None.
      """
        encrypt1 = EncryptConfig("gpg", "Backup User")
        encrypt2 = EncryptConfig("gpg", "Backup User")
        self.assertEqual(encrypt1, encrypt2)
        self.assertTrue(encrypt1 == encrypt2)
        self.assertTrue(not encrypt1 < encrypt2)
        self.assertTrue(encrypt1 <= encrypt2)
        self.assertTrue(not encrypt1 > encrypt2)
        self.assertTrue(encrypt1 >= encrypt2)
        self.assertTrue(not encrypt1 != encrypt2)

    def testComparison_003(self):
        """
      Test comparison of two differing objects, encryptMode differs (one None).
      """
        encrypt1 = EncryptConfig()
        encrypt2 = EncryptConfig(encryptMode="gpg")
        self.assertNotEqual(encrypt1, encrypt2)
        self.assertTrue(not encrypt1 == encrypt2)
        self.assertTrue(encrypt1 < encrypt2)
        self.assertTrue(encrypt1 <= encrypt2)
        self.assertTrue(not encrypt1 > encrypt2)
        self.assertTrue(not encrypt1 >= encrypt2)
        self.assertTrue(encrypt1 != encrypt2)

    # Note: no test to check when encrypt mode differs, since only one value is allowed

    def testComparison_004(self):
        """
      Test comparison of two differing objects, encryptTarget differs (one None).
      """
        encrypt1 = EncryptConfig()
        encrypt2 = EncryptConfig(encryptTarget="Backup User")
        self.assertNotEqual(encrypt1, encrypt2)
        self.assertTrue(not encrypt1 == encrypt2)
        self.assertTrue(encrypt1 < encrypt2)
        self.assertTrue(encrypt1 <= encrypt2)
        self.assertTrue(not encrypt1 > encrypt2)
        self.assertTrue(not encrypt1 >= encrypt2)
        self.assertTrue(encrypt1 != encrypt2)

    def testComparison_005(self):
        """
      Test comparison of two differing objects, encryptTarget differs.
      """
        encrypt1 = EncryptConfig("gpg", "Another User")
        encrypt2 = EncryptConfig("gpg", "Backup User")
        self.assertNotEqual(encrypt1, encrypt2)
        self.assertTrue(not encrypt1 == encrypt2)
        self.assertTrue(encrypt1 < encrypt2)
        self.assertTrue(encrypt1 <= encrypt2)
        self.assertTrue(not encrypt1 > encrypt2)
        self.assertTrue(not encrypt1 >= encrypt2)
        self.assertTrue(encrypt1 != encrypt2)


########################
# TestLocalConfig class
########################


class TestLocalConfig(unittest.TestCase):

    """Tests for the LocalConfig class."""

    ################
    # Setup methods
    ################

    @classmethod
    def setUpClass(cls):
        configureLogging()

    def setUp(self):
        try:
            self.resources = findResources(RESOURCES, DATA_DIRS)
        except Exception as e:
            self.fail(e)

    def tearDown(self):
        pass

    ##################
    # Utility methods
    ##################

    def failUnlessAssignRaises(self, exception, obj, prop, value):
        """Equivalent of :any:`failUnlessRaises`, but used for property assignments instead."""
        failUnlessAssignRaises(self, exception, obj, prop, value)

    def validateAddConfig(self, origConfig):
        """
      Validates that document dumped from ``LocalConfig.addConfig`` results in
      identical object.

      We dump a document containing just the encrypt configuration, and then
      make sure that if we push that document back into the ``LocalConfig``
      object, that the resulting object matches the original.

      The ``self.failUnlessEqual`` method is used for the validation, so if the
      method call returns normally, everything is OK.

      Args:
         origConfig: Original configuration
      """
        (xmlDom, parentNode) = createOutputDom()
        origConfig.addConfig(xmlDom, parentNode)
        xmlData = serializeDom(xmlDom)
        newConfig = LocalConfig(xmlData=xmlData, validate=False)
        self.assertEqual(origConfig, newConfig)

    ############################
    # Test __repr__ and __str__
    ############################

    def testStringFuncs_001(self):
        """
      Just make sure that the string functions don't have errors (i.e. bad variable names).
      """
        obj = LocalConfig()
        obj.__repr__()
        obj.__str__()

    #####################################################
    # Test basic constructor and attribute functionality
    #####################################################

    def testConstructor_001(self):
        """
      Test empty constructor, validate=False.
      """
        config = LocalConfig(validate=False)
        self.assertEqual(None, config.encrypt)

    def testConstructor_002(self):
        """
      Test empty constructor, validate=True.
      """
        config = LocalConfig(validate=True)
        self.assertEqual(None, config.encrypt)

    def testConstructor_003(self):
        """
      Test with empty config document as both data and file, validate=False.
      """
        path = self.resources["encrypt.conf.1"]
        with open(path) as f:
            contents = f.read()
        self.assertRaises(ValueError, LocalConfig, xmlData=contents, xmlPath=path, validate=False)

    def testConstructor_004(self):
        """
      Test assignment of encrypt attribute, None value.
      """
        config = LocalConfig()
        config.encrypt = None
        self.assertEqual(None, config.encrypt)

    def testConstructor_005(self):
        """
      Test assignment of encrypt attribute, valid value.
      """
        config = LocalConfig()
        config.encrypt = EncryptConfig()
        self.assertEqual(EncryptConfig(), config.encrypt)

    def testConstructor_006(self):
        """
      Test assignment of encrypt attribute, invalid value (not EncryptConfig).
      """
        config = LocalConfig()
        self.failUnlessAssignRaises(ValueError, config, "encrypt", "STRING!")

    ############################
    # Test comparison operators
    ############################

    def testComparison_001(self):
        """
      Test comparison of two identical objects, all attributes None.
      """
        config1 = LocalConfig()
        config2 = LocalConfig()
        self.assertEqual(config1, config2)
        self.assertTrue(config1 == config2)
        self.assertTrue(not config1 < config2)
        self.assertTrue(config1 <= config2)
        self.assertTrue(not config1 > config2)
        self.assertTrue(config1 >= config2)
        self.assertTrue(not config1 != config2)

    def testComparison_002(self):
        """
      Test comparison of two identical objects, all attributes non-None.
      """
        config1 = LocalConfig()
        config1.encrypt = EncryptConfig()

        config2 = LocalConfig()
        config2.encrypt = EncryptConfig()

        self.assertEqual(config1, config2)
        self.assertTrue(config1 == config2)
        self.assertTrue(not config1 < config2)
        self.assertTrue(config1 <= config2)
        self.assertTrue(not config1 > config2)
        self.assertTrue(config1 >= config2)
        self.assertTrue(not config1 != config2)

    def testComparison_003(self):
        """
      Test comparison of two differing objects, encrypt differs (one None).
      """
        config1 = LocalConfig()
        config2 = LocalConfig()
        config2.encrypt = EncryptConfig()
        self.assertNotEqual(config1, config2)
        self.assertTrue(not config1 == config2)
        self.assertTrue(config1 < config2)
        self.assertTrue(config1 <= config2)
        self.assertTrue(not config1 > config2)
        self.assertTrue(not config1 >= config2)
        self.assertTrue(config1 != config2)

    def testComparison_004(self):
        """
      Test comparison of two differing objects, encrypt differs.
      """
        config1 = LocalConfig()
        config1.encrypt = EncryptConfig(encryptTarget="Another User")

        config2 = LocalConfig()
        config2.encrypt = EncryptConfig(encryptTarget="Backup User")

        self.assertNotEqual(config1, config2)
        self.assertTrue(not config1 == config2)
        self.assertTrue(config1 < config2)
        self.assertTrue(config1 <= config2)
        self.assertTrue(not config1 > config2)
        self.assertTrue(not config1 >= config2)
        self.assertTrue(config1 != config2)

    ######################
    # Test validate logic
    ######################

    def testValidate_001(self):
        """
      Test validate on a None encrypt section.
      """
        config = LocalConfig()
        config.encrypt = None
        self.assertRaises(ValueError, config.validate)

    def testValidate_002(self):
        """
      Test validate on an empty encrypt section.
      """
        config = LocalConfig()
        config.encrypt = EncryptConfig()
        self.assertRaises(ValueError, config.validate)

    def testValidate_003(self):
        """
      Test validate on a non-empty encrypt section with no values filled in.
      """
        config = LocalConfig()
        config.encrypt = EncryptConfig(None, None)
        self.assertRaises(ValueError, config.validate)

    def testValidate_004(self):
        """
      Test validate on a non-empty encrypt section with only one value filled in.
      """
        config = LocalConfig()
        config.encrypt = EncryptConfig("gpg", None)
        self.assertRaises(ValueError, config.validate)
        config.encrypt = EncryptConfig(None, "Backup User")
        self.assertRaises(ValueError, config.validate)

    def testValidate_005(self):
        """
      Test validate on a non-empty encrypt section with valid values filled in.
      """
        config = LocalConfig()
        config.encrypt = EncryptConfig("gpg", "Backup User")
        config.validate()

    ############################
    # Test parsing of documents
    ############################

    def testParse_001(self):
        """
      Parse empty config document.
      """
        path = self.resources["encrypt.conf.1"]
        with open(path) as f:
            contents = f.read()
        self.assertRaises(ValueError, LocalConfig, xmlPath=path, validate=True)
        self.assertRaises(ValueError, LocalConfig, xmlData=contents, validate=True)
        config = LocalConfig(xmlPath=path, validate=False)
        self.assertEqual(None, config.encrypt)
        config = LocalConfig(xmlData=contents, validate=False)
        self.assertEqual(None, config.encrypt)

    def testParse_002(self):
        """
      Parse config document with filled-in values.
      """
        path = self.resources["encrypt.conf.2"]
        with open(path) as f:
            contents = f.read()
        config = LocalConfig(xmlPath=path, validate=False)
        self.assertNotEqual(None, config.encrypt)
        self.assertEqual("gpg", config.encrypt.encryptMode)
        self.assertEqual("Backup User", config.encrypt.encryptTarget)
        config = LocalConfig(xmlData=contents, validate=False)
        self.assertNotEqual(None, config.encrypt)
        self.assertEqual("gpg", config.encrypt.encryptMode)
        self.assertEqual("Backup User", config.encrypt.encryptTarget)

    ###################
    # Test addConfig()
    ###################

    def testAddConfig_001(self):
        """
      Test with empty config document.
      """
        encrypt = EncryptConfig()
        config = LocalConfig()
        config.encrypt = encrypt
        self.validateAddConfig(config)

    def testAddConfig_002(self):
        """
      Test with values set.
      """
        encrypt = EncryptConfig(encryptMode="gpg", encryptTarget="Backup User")
        config = LocalConfig()
        config.encrypt = encrypt
        self.validateAddConfig(config)


######################
# TestFunctions class
######################


@unittest.skipUnless(runAllTests(), "Limited test suite")
class TestFunctions(unittest.TestCase):

    """Tests for the functions in encrypt.py."""

    ################
    # Setup methods
    ################

    @classmethod
    def setUpClass(cls):
        configureLogging()

    def setUp(self):
        try:
            self.tmpdir = tempfile.mkdtemp()
            self.resources = findResources(RESOURCES, DATA_DIRS)
        except Exception as e:
            self.fail(e)

    def tearDown(self):
        try:
            removedir(self.tmpdir)
        except:
            pass

    ##################
    # Utility methods
    ##################

    def extractTar(self, tarname):
        """Extracts a tarfile with a particular name."""
        extractTar(self.tmpdir, self.resources["%s.tar.gz" % tarname])

    def buildPath(self, components):
        """Builds a complete search path from a list of components."""
        components.insert(0, self.tmpdir)
        return buildPath(components)

    #############################
    # Test _encryptFileWithGpg()
    #############################

    def testEncryptFileWithGpg_001(self):
        """
      Test for a non-existent file in a non-existent directory.
      """
        sourceFile = self.buildPath([INVALID_PATH, INVALID_PATH])
        self.assertRaises(IOError, _encryptFileWithGpg, sourceFile, INVALID_GPG_RECIPIENT)

    def testEncryptFileWithGpg_002(self):
        """
      Test for a non-existent file in an existing directory.
      """
        self.extractTar("tree8")
        sourceFile = self.buildPath(["tree8", "dir001", INVALID_PATH,])
        self.assertRaises(IOError, _encryptFileWithGpg, sourceFile, INVALID_GPG_RECIPIENT)

    def testEncryptFileWithGpg_003(self):
        """
      Test for an unknown recipient.
      """
        self.extractTar("tree1")
        sourceFile = self.buildPath(["tree1", "file001"])
        expectedFile = self.buildPath(["tree1", "file001.gpg"])
        self.assertRaises(IOError, _encryptFileWithGpg, sourceFile, INVALID_GPG_RECIPIENT)
        self.assertFalse(os.path.exists(expectedFile))
        self.assertTrue(os.path.exists(sourceFile))

    def testEncryptFileWithGpg_004(self):
        """
      Test for a valid recipient.
      """
        self.extractTar("tree1")
        sourceFile = self.buildPath(["tree1", "file001"])
        expectedFile = self.buildPath(["tree1", "file001.gpg"])
        actualFile = _encryptFileWithGpg(sourceFile, VALID_GPG_RECIPIENT)
        self.assertEqual(actualFile, expectedFile)
        self.assertTrue(os.path.exists(sourceFile))
        self.assertTrue(os.path.exists(actualFile))

    ######################
    # Test _encryptFile()
    ######################

    def testEncryptFile_001(self):
        """
      Test for a mode other than "gpg".
      """
        self.extractTar("tree1")
        sourceFile = self.buildPath(["tree1", "file001"])
        expectedFile = self.buildPath(["tree1", "file001.gpg"])
        self.assertRaises(ValueError, _encryptFile, sourceFile, "pgp", INVALID_GPG_RECIPIENT, None, None, removeSource=False)
        self.assertTrue(os.path.exists(sourceFile))
        self.assertFalse(os.path.exists(expectedFile))

    def testEncryptFile_002(self):
        """
      Test for a source path that does not exist.
      """
        self.extractTar("tree1")
        sourceFile = self.buildPath(["tree1", INVALID_PATH])
        expectedFile = self.buildPath(["tree1", "%s.gpg" % INVALID_PATH])
        self.assertRaises(ValueError, _encryptFile, sourceFile, "gpg", INVALID_GPG_RECIPIENT, None, None, removeSource=False)
        self.assertFalse(os.path.exists(sourceFile))
        self.assertFalse(os.path.exists(expectedFile))

    def testEncryptFile_003(self):
        """
      Test "gpg" mode with a valid source path and invalid recipient, removeSource=False.
      """
        self.extractTar("tree1")
        sourceFile = self.buildPath(["tree1", "file001"])
        expectedFile = self.buildPath(["tree1", "file001.gpg"])
        self.assertRaises(IOError, _encryptFile, sourceFile, "gpg", INVALID_GPG_RECIPIENT, None, None, removeSource=False)
        self.assertTrue(os.path.exists(sourceFile))
        self.assertFalse(os.path.exists(expectedFile))

    def testEncryptFile_004(self):
        """
      Test "gpg" mode with a valid source path and invalid recipient, removeSource=True.
      """
        self.extractTar("tree1")
        sourceFile = self.buildPath(["tree1", "file001"])
        expectedFile = self.buildPath(["tree1", "file001.gpg"])
        self.assertRaises(IOError, _encryptFile, sourceFile, "gpg", INVALID_GPG_RECIPIENT, None, None, removeSource=True)
        self.assertTrue(os.path.exists(sourceFile))
        self.assertFalse(os.path.exists(expectedFile))

    def testEncryptFile_005(self):
        """
      Test "gpg" mode with a valid source path and recipient, removeSource=False.
      """
        self.extractTar("tree1")
        sourceFile = self.buildPath(["tree1", "file001"])
        expectedFile = self.buildPath(["tree1", "file001.gpg"])
        actualFile = _encryptFile(sourceFile, "gpg", VALID_GPG_RECIPIENT, None, None, removeSource=False)
        self.assertEqual(actualFile, expectedFile)
        self.assertTrue(os.path.exists(sourceFile))
        self.assertTrue(os.path.exists(actualFile))

    def testEncryptFile_006(self):
        """
      Test "gpg" mode with a valid source path and recipient, removeSource=True.
      """
        self.extractTar("tree1")
        sourceFile = self.buildPath(["tree1", "file001"])
        expectedFile = self.buildPath(["tree1", "file001.gpg"])
        actualFile = _encryptFile(sourceFile, "gpg", VALID_GPG_RECIPIENT, None, None, removeSource=True)
        self.assertEqual(actualFile, expectedFile)
        self.assertFalse(os.path.exists(sourceFile))
        self.assertTrue(os.path.exists(actualFile))

    ##########################
    # Test _encryptDailyDir()
    ##########################

    def testEncryptDailyDir_001(self):
        """
      Test with a nonexistent daily staging directory.
      """
        self.extractTar("tree1")
        dailyDir = self.buildPath(["tree1", "dir001"])
        self.assertRaises(ValueError, _encryptDailyDir, dailyDir, "gpg", VALID_GPG_RECIPIENT, None, None)

    def testEncryptDailyDir_002(self):
        """
      Test with a valid staging directory containing only links.
      """
        self.extractTar("tree15")
        dailyDir = self.buildPath(["tree15", "dir001"])
        fsList = FilesystemList()
        fsList.addDirContents(dailyDir)
        self.assertEqual(3, len(fsList))
        self.assertTrue(self.buildPath(["tree15", "dir001",]) in fsList)
        self.assertTrue(self.buildPath(["tree15", "dir001", "link001",]) in fsList)
        self.assertTrue(self.buildPath(["tree15", "dir001", "link002",]) in fsList)
        _encryptDailyDir(dailyDir, "gpg", VALID_GPG_RECIPIENT, None, None)
        fsList = FilesystemList()
        fsList.addDirContents(dailyDir)
        self.assertEqual(3, len(fsList))
        self.assertTrue(self.buildPath(["tree15", "dir001",]) in fsList)
        self.assertTrue(self.buildPath(["tree15", "dir001", "link001",]) in fsList)
        self.assertTrue(self.buildPath(["tree15", "dir001", "link002",]) in fsList)

    def testEncryptDailyDir_003(self):
        """
      Test with a valid staging directory containing only directories.
      """
        self.extractTar("tree2")
        dailyDir = self.buildPath(["tree2"])
        fsList = FilesystemList()
        fsList.addDirContents(dailyDir)
        self.assertEqual(11, len(fsList))
        self.assertTrue(self.buildPath(["tree2",]) in fsList)
        self.assertTrue(self.buildPath(["tree2", "dir001",]) in fsList)
        self.assertTrue(self.buildPath(["tree2", "dir002",]) in fsList)
        self.assertTrue(self.buildPath(["tree2", "dir003",]) in fsList)
        self.assertTrue(self.buildPath(["tree2", "dir004",]) in fsList)
        self.assertTrue(self.buildPath(["tree2", "dir005",]) in fsList)
        self.assertTrue(self.buildPath(["tree2", "dir006",]) in fsList)
        self.assertTrue(self.buildPath(["tree2", "dir007",]) in fsList)
        self.assertTrue(self.buildPath(["tree2", "dir008",]) in fsList)
        self.assertTrue(self.buildPath(["tree2", "dir009",]) in fsList)
        self.assertTrue(self.buildPath(["tree2", "dir010",]) in fsList)
        _encryptDailyDir(dailyDir, "gpg", VALID_GPG_RECIPIENT, None, None)
        fsList = FilesystemList()
        fsList.addDirContents(dailyDir)
        self.assertEqual(11, len(fsList))
        self.assertTrue(self.buildPath(["tree2",]) in fsList)
        self.assertTrue(self.buildPath(["tree2", "dir001",]) in fsList)
        self.assertTrue(self.buildPath(["tree2", "dir002",]) in fsList)
        self.assertTrue(self.buildPath(["tree2", "dir003",]) in fsList)
        self.assertTrue(self.buildPath(["tree2", "dir004",]) in fsList)
        self.assertTrue(self.buildPath(["tree2", "dir005",]) in fsList)
        self.assertTrue(self.buildPath(["tree2", "dir006",]) in fsList)
        self.assertTrue(self.buildPath(["tree2", "dir007",]) in fsList)
        self.assertTrue(self.buildPath(["tree2", "dir008",]) in fsList)
        self.assertTrue(self.buildPath(["tree2", "dir009",]) in fsList)
        self.assertTrue(self.buildPath(["tree2", "dir010",]) in fsList)

    def testEncryptDailyDir_004(self):
        """
      Test with a valid staging directory containing only files.
      """
        self.extractTar("tree1")
        dailyDir = self.buildPath(["tree1"])
        fsList = FilesystemList()
        fsList.addDirContents(dailyDir)
        self.assertEqual(8, len(fsList))
        self.assertTrue(self.buildPath(["tree1"]) in fsList)
        self.assertTrue(self.buildPath(["tree1", "file001",]) in fsList)
        self.assertTrue(self.buildPath(["tree1", "file002",]) in fsList)
        self.assertTrue(self.buildPath(["tree1", "file003",]) in fsList)
        self.assertTrue(self.buildPath(["tree1", "file004",]) in fsList)
        self.assertTrue(self.buildPath(["tree1", "file005",]) in fsList)
        self.assertTrue(self.buildPath(["tree1", "file006",]) in fsList)
        self.assertTrue(self.buildPath(["tree1", "file007",]) in fsList)
        _encryptDailyDir(dailyDir, "gpg", VALID_GPG_RECIPIENT, None, None)
        fsList = FilesystemList()
        fsList.addDirContents(dailyDir)
        self.assertEqual(8, len(fsList))
        self.assertTrue(self.buildPath(["tree1"]) in fsList)
        self.assertTrue(self.buildPath(["tree1", "file001.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree1", "file002.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree1", "file003.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree1", "file004.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree1", "file005.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree1", "file006.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree1", "file007.gpg",]) in fsList)

    def testEncryptDailyDir_005(self):
        """
      Test with a valid staging directory containing files, directories and
      links, including various files that match the general Cedar Backup
      indicator file pattern ("cback.<something>").
      """
        self.extractTar("tree16")
        dailyDir = self.buildPath(["tree16"])
        fsList = FilesystemList()
        fsList.addDirContents(dailyDir)
        self.assertEqual(122, len(fsList))
        self.assertTrue(self.buildPath(["tree16",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir001", "file001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir001", "file002",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir001", "file003",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir001", "file004",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir001", "file005",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir001", "file006",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir001", "file007",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir001", "file008",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir001", "link001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir002",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir002", "file001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir002", "file002",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir002", "file003",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir002", "file004",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir002", "link001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir003",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir003", "file001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir003", "file002",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir003", "file003",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir003", "link001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir003", "link002",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002", "dir001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002", "dir001", "cback.encrypt",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002", "dir001", "file001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002", "dir001", "file002",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002", "dir001", "file003",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002", "dir001", "link001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002", "dir002",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002", "dir002", "cback.encrypt",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002", "dir002", "file001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002", "dir002", "file002",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002", "dir002", "file003",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002", "dir002", "file004",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002", "dir002", "file005",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002", "dir002", "link001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002", "dir002", "link002",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002", "dir002", "link003",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002", "dir002", "link004",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002", "dir002", "link005",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir001", "file001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir001", "file002",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir001", "file003",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir001", "file004",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir001", "file005",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir001", "file006",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir001", "file007",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir001", "file008",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir001", "link001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir001", "cback.",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir002",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir002", "file001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir002", "file002",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir002", "file003",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir002", "file004",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir002", "link001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir002", "cback.encrypt",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir003",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir003", "file001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir003", "file002",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir003", "file003",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir003", "link001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir003", "link002",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir003", "cback.encrypt",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir003", "cback.store",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir001", "file001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir001", "file002",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir001", "file003",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir001", "file004",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir001", "file005",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir001", "file006",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir001", "file007",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir001", "file008",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir001", "link001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir001", "cback.",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir001", "cback.collect",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir002",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir002", "file001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir002", "file002",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir002", "file003",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir002", "file004",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir002", "link001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir002", "cback.encrypt",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir003",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir003", "file001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir003", "file002",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir003", "file003",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir003", "link001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir003", "link002",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir003", "cback.encrypt",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "cback.encrypt",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir004",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir004", "file001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir004", "file002",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir004", "file003",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir004", "file004",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir004", "file005",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir004", "file006",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir004", "file007",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir004", "file008",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir004", "link001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir004", "cback.store",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir005",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir005", "file001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir005", "file002",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir005", "file003",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir005", "file004",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir005", "file005",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir005", "file006",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir005", "file007",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir005", "file008",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir005", "link001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "cback.collect",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "cback.stage",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "cback.store",]) in fsList)
        _encryptDailyDir(dailyDir, "gpg", VALID_GPG_RECIPIENT, None, None)
        fsList = FilesystemList()
        fsList.addDirContents(dailyDir)
        # since all links are to files, and the files all changed names, the links are invalid and disappear
        self.assertEqual(102, len(fsList))
        self.assertTrue(self.buildPath(["tree16",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir001", "file001.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir001", "file002.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir001", "file003.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir001", "file004.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir001", "file005.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir001", "file006.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir001", "file007.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir001", "file008.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir002",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir002", "file001.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir002", "file002.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir002", "file003.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir002", "file004.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir003",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir003", "file001.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir003", "file002.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir001", "dir003", "file003.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002", "dir001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002", "dir001", "cback.encrypt",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002", "dir001", "file001.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002", "dir001", "file002.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002", "dir001", "file003.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002", "dir002",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002", "dir002", "cback.encrypt",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002", "dir002", "file001.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002", "dir002", "file002.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002", "dir002", "file003.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002", "dir002", "file004.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir002", "dir002", "file005.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir001", "file001.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir001", "file002.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir001", "file003.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir001", "file004.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir001", "file005.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir001", "file006.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir001", "file007.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir001", "file008.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir001", "cback.",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir002",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir002", "file001.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir002", "file002.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir002", "file003.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir002", "file004.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir002", "cback.encrypt",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir003",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir003", "file001.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir003", "file002.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir003", "file003.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir003", "cback.encrypt",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir003", "dir003", "cback.store",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir001",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir001", "file001.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir001", "file002.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir001", "file003.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir001", "file004.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir001", "file005.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir001", "file006.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir001", "file007.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir001", "file008.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir001", "cback.",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir001", "cback.collect",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir002",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir002", "file001.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir002", "file002.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir002", "file003.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir002", "file004.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir002", "cback.encrypt",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir003",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir003", "file001.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir003", "file002.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir003", "file003.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir003", "cback.encrypt",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "cback.encrypt",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir004",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir004", "file001.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir004", "file002.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir004", "file003.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir004", "file004.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir004", "file005.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir004", "file006.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir004", "file007.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir004", "file008.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir004", "cback.store",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir005",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir005", "file001.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir005", "file002.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir005", "file003.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir005", "file004.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir005", "file005.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir005", "file006.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir005", "file007.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "dir004", "dir005", "file008.gpg",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "cback.collect",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "cback.stage",]) in fsList)
        self.assertTrue(self.buildPath(["tree16", "cback.store",]) in fsList)



# -*- coding: iso-8859-1 -*-
# vim: set ft=python ts=4 sw=4 expandtab:
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
#              C E D A R
#          S O L U T I O N S       "Software done right."
#           S O F T W A R E
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
# Copyright (c) 2006,2010,2015 Kenneth J. Pronovici.
# All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License,
# Version 2, as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# Copies of the GNU General Public License are available from
# the Free Software Foundation website, http://www.gnu.org/.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
# Author   : Kenneth J. Pronovici <pronovic@ieee.org>
# Language : Python 3 (>= 3.7)
# Project  : Cedar Backup, release 3
# Purpose  : Tests PostgreSQL extension functionality.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

########################################################################
# Module documentation
########################################################################

"""
Unit tests for CedarBackup3/extend/postgresql.py.

Code Coverage
=============

   This module contains individual tests for the many of the public functions
   and classes implemented in extend/postgresql.py.  There are also tests for
   several of the private methods.

   Unfortunately, it's rather difficult to test this code in an automated
   fashion, even if you have access to PostgreSQL, since the actual dump would
   need to have access to a real database.  Because of this, there aren't any
   tests below that actually talk to a database.

   As a compromise, I test some of the private methods in the implementation.
   Normally, I don't like to test private methods, but in this case, testing
   the private methods will help give us some reasonable confidence in the code
   even if we can't talk to a database..  This isn't perfect, but it's better
   than nothing.

Naming Conventions
==================

   I prefer to avoid large unit tests which validate more than one piece of
   functionality, and I prefer to avoid using overly descriptive (read: long)
   test names, as well.  Instead, I use lots of very small tests that each
   validate one specific thing.  These small tests are then named with an index
   number, yielding something like ``testAddDir_001`` or ``testValidate_010``.
   Each method has a docstring describing what it's supposed to accomplish.  I
   feel that this makes it easier to judge how important a given failure is,
   and also makes it somewhat easier to diagnose and fix individual problems.

Testing XML Extraction
======================

   It's difficult to validated that generated XML is exactly "right",
   especially when dealing with pretty-printed XML.  We can't just provide a
   constant string and say "the result must match this".  Instead, what we do
   is extract a node, build some XML from it, and then feed that XML back into
   another object's constructor.  If that parse process succeeds and the old
   object is equal to the new object, we assume that the extract was
   successful.

   It would arguably be better if we could do a completely independent check -
   but implementing that check would be equivalent to re-implementing all of
   the existing functionality that we're validating here!  After all, the most
   important thing is that data can move seamlessly from object to XML document
   and back to object.

Full vs. Reduced Tests
======================

   All of the tests in this module are considered safe to be run in an average
   build environment.  There is a no need to use a POSTGRESQLTESTS_FULL
   environment variable to provide a "reduced feature set" test suite as for
   some of the other test modules.

@author Kenneth J. Pronovici <pronovic@ieee.org>
"""


########################################################################
# Import modules and do runtime validations
########################################################################

import unittest

from CedarBackup3.extend.postgresql import LocalConfig, PostgresqlConfig
from CedarBackup3.testutil import configureLogging, failUnlessAssignRaises, findResources
from CedarBackup3.xmlutil import createOutputDom, serializeDom

#######################################################################
# Module-wide configuration and constants
#######################################################################

DATA_DIRS = [
    "./data",
    "./tests/data",
]
RESOURCES = [
    "postgresql.conf.1",
    "postgresql.conf.2",
    "postgresql.conf.3",
    "postgresql.conf.4",
    "postgresql.conf.5",
]


#######################################################################
# Test Case Classes
#######################################################################

#############################
# TestPostgresqlConfig class
#############################


class TestPostgresqlConfig(unittest.TestCase):

    """Tests for the PostgresqlConfig class."""

    ################
    # Setup methods
    ################

    @classmethod
    def setUpClass(cls):
        configureLogging()

    ##################
    # Utility methods
    ##################

    def failUnlessAssignRaises(self, exception, obj, prop, value):
        """Equivalent of :any:`failUnlessRaises`, but used for property assignments instead."""
        failUnlessAssignRaises(self, exception, obj, prop, value)

    ############################
    # Test __repr__ and __str__
    ############################

    def testStringFuncs_001(self):
        """
      Just make sure that the string functions don't have errors (i.e. bad variable names).
      """
        obj = PostgresqlConfig()
        obj.__repr__()
        obj.__str__()

    ##################################
    # Test constructor and attributes
    ##################################

    def testConstructor_001(self):
        """
      Test constructor with no values filled in.
      """
        postgresql = PostgresqlConfig()
        self.assertEqual(None, postgresql.user)
        self.assertEqual(None, postgresql.compressMode)
        self.assertEqual(False, postgresql.all)
        self.assertEqual(None, postgresql.databases)

    def testConstructor_002(self):
        """
      Test constructor with all values filled in, with valid values, databases=None.
      """
        postgresql = PostgresqlConfig("user", "none", False, None)
        self.assertEqual("user", postgresql.user)
        self.assertEqual("none", postgresql.compressMode)
        self.assertEqual(False, postgresql.all)
        self.assertEqual(None, postgresql.databases)

    def testConstructor_003(self):
        """
      Test constructor with all values filled in, with valid values, no databases.
      """
        postgresql = PostgresqlConfig("user", "none", True, [])
        self.assertEqual("user", postgresql.user)
        self.assertEqual("none", postgresql.compressMode)
        self.assertEqual(True, postgresql.all)
        self.assertEqual([], postgresql.databases)

    def testConstructor_004(self):
        """
      Test constructor with all values filled in, with valid values, with one database.
      """
        postgresql = PostgresqlConfig("user", "gzip", True, ["one",])
        self.assertEqual("user", postgresql.user)
        self.assertEqual("gzip", postgresql.compressMode)
        self.assertEqual(True, postgresql.all)
        self.assertEqual(["one",], postgresql.databases)

    def testConstructor_005(self):
        """
      Test constructor with all values filled in, with valid values, with multiple databases.
      """
        postgresql = PostgresqlConfig("user", "bzip2", True, ["one", "two",])
        self.assertEqual("user", postgresql.user)
        self.assertEqual("bzip2", postgresql.compressMode)
        self.assertEqual(True, postgresql.all)
        self.assertEqual(["one", "two",], postgresql.databases)

    def testConstructor_006(self):
        """
      Test assignment of user attribute, None value.
      """
        postgresql = PostgresqlConfig(user="user")
        self.assertEqual("user", postgresql.user)
        postgresql.user = None
        self.assertEqual(None, postgresql.user)

    def testConstructor_007(self):
        """
      Test assignment of user attribute, valid value.
      """
        postgresql = PostgresqlConfig()
        self.assertEqual(None, postgresql.user)
        postgresql.user = "user"
        self.assertEqual("user", postgresql.user)

    def testConstructor_008(self):
        """
      Test assignment of user attribute, invalid value (empty).
      """
        postgresql = PostgresqlConfig()
        self.assertEqual(None, postgresql.user)
        self.failUnlessAssignRaises(ValueError, postgresql, "user", "")
        self.assertEqual(None, postgresql.user)

    def testConstructor_009(self):
        """
      Test assignment of compressMode attribute, None value.
      """
        postgresql = PostgresqlConfig(compressMode="none")
        self.assertEqual("none", postgresql.compressMode)
        postgresql.compressMode = None
        self.assertEqual(None, postgresql.compressMode)

    def testConstructor_010(self):
        """
      Test assignment of compressMode attribute, valid value.
      """
        postgresql = PostgresqlConfig()
        self.assertEqual(None, postgresql.compressMode)
        postgresql.compressMode = "none"
        self.assertEqual("none", postgresql.compressMode)
        postgresql.compressMode = "gzip"
        self.assertEqual("gzip", postgresql.compressMode)
        postgresql.compressMode = "bzip2"
        self.assertEqual("bzip2", postgresql.compressMode)

    def testConstructor_011(self):
        """
      Test assignment of compressMode attribute, invalid value (empty).
      """
        postgresql = PostgresqlConfig()
        self.assertEqual(None, postgresql.compressMode)
        self.failUnlessAssignRaises(ValueError, postgresql, "compressMode", "")
        self.assertEqual(None, postgresql.compressMode)

    def testConstructor_012(self):
        """
      Test assignment of compressMode attribute, invalid value (not in list).
      """
        postgresql = PostgresqlConfig()
        self.assertEqual(None, postgresql.compressMode)
        self.failUnlessAssignRaises(ValueError, postgresql, "compressMode", "bogus")
        self.assertEqual(None, postgresql.compressMode)

    def testConstructor_013(self):
        """
      Test assignment of all attribute, None value.
      """
        postgresql = PostgresqlConfig(all=True)
        self.assertEqual(True, postgresql.all)
        postgresql.all = None
        self.assertEqual(False, postgresql.all)

    def testConstructor_014(self):
        """
      Test assignment of all attribute, valid value (real boolean).
      """
        postgresql = PostgresqlConfig()
        self.assertEqual(False, postgresql.all)
        postgresql.all = True
        self.assertEqual(True, postgresql.all)
        postgresql.all = False
        self.assertEqual(False, postgresql.all)

    def testConstructor_015(self):
        """
      Test assignment of all attribute, valid value (expression).
      """
        postgresql = PostgresqlConfig()
        self.assertEqual(False, postgresql.all)
        postgresql.all = 0
        self.assertEqual(False, postgresql.all)
        postgresql.all = []
        self.assertEqual(False, postgresql.all)
        postgresql.all = None
        self.assertEqual(False, postgresql.all)
        postgresql.all = ["a"]
        self.assertEqual(True, postgresql.all)
        postgresql.all = 3
        self.assertEqual(True, postgresql.all)

    def testConstructor_016(self):
        """
      Test assignment of databases attribute, None value.
      """
        postgresql = PostgresqlConfig(databases=[])
        self.assertEqual([], postgresql.databases)
        postgresql.databases = None
        self.assertEqual(None, postgresql.databases)

    def testConstructor_017(self):
        """
      Test assignment of databases attribute, [] value.
      """
        postgresql = PostgresqlConfig()
        self.assertEqual(None, postgresql.databases)
        postgresql.databases = []
        self.assertEqual([], postgresql.databases)

    def testConstructor_018(self):
        """
      Test assignment of databases attribute, single valid entry.
      """
        postgresql = PostgresqlConfig()
        self.assertEqual(None, postgresql.databases)
        postgresql.databases = [
            "/whatever",
        ]
        self.assertEqual(["/whatever",], postgresql.databases)
        postgresql.databases.append("/stuff")
        self.assertEqual(["/whatever", "/stuff",], postgresql.databases)

    def testConstructor_019(self):
        """
      Test assignment of databases attribute, multiple valid entries.
      """
        postgresql = PostgresqlConfig()
        self.assertEqual(None, postgresql.databases)
        postgresql.databases = [
            "/whatever",
            "/stuff",
        ]
        self.assertEqual(["/whatever", "/stuff",], postgresql.databases)
        postgresql.databases.append("/etc/X11")
        self.assertEqual(["/whatever", "/stuff", "/etc/X11",], postgresql.databases)

    def testConstructor_020(self):
        """
      Test assignment of databases attribute, single invalid entry (empty).
      """
        postgresql = PostgresqlConfig()
        self.assertEqual(None, postgresql.databases)
        self.failUnlessAssignRaises(ValueError, postgresql, "databases", ["",])
        self.assertEqual(None, postgresql.databases)

    def testConstructor_021(self):
        """
      Test assignment of databases attribute, mixed valid and invalid entries.
      """
        postgresql = PostgresqlConfig()
        self.assertEqual(None, postgresql.databases)
        self.failUnlessAssignRaises(ValueError, postgresql, "databases", ["good", "", "alsogood",])
        self.assertEqual(None, postgresql.databases)

    ############################
    # Test comparison operators
    ############################

    def testComparison_001(self):
        """
      Test comparison of two identical objects, all attributes None.
      """
        postgresql1 = PostgresqlConfig()
        postgresql2 = PostgresqlConfig()
        self.assertEqual(postgresql1, postgresql2)
        self.assertTrue(postgresql1 == postgresql2)
        self.assertTrue(not postgresql1 < postgresql2)
        self.assertTrue(postgresql1 <= postgresql2)
        self.assertTrue(not postgresql1 > postgresql2)
        self.assertTrue(postgresql1 >= postgresql2)
        self.assertTrue(not postgresql1 != postgresql2)

    def testComparison_002(self):
        """
      Test comparison of two identical objects, all attributes non-None, list None.
      """
        postgresql1 = PostgresqlConfig("user", "gzip", True, None)
        postgresql2 = PostgresqlConfig("user", "gzip", True, None)
        self.assertEqual(postgresql1, postgresql2)
        self.assertTrue(postgresql1 == postgresql2)
        self.assertTrue(not postgresql1 < postgresql2)
        self.assertTrue(postgresql1 <= postgresql2)
        self.assertTrue(not postgresql1 > postgresql2)
        self.assertTrue(postgresql1 >= postgresql2)
        self.assertTrue(not postgresql1 != postgresql2)

    def testComparison_003(self):
        """
      Test comparison of two identical objects, all attributes non-None, list empty.
      """
        postgresql1 = PostgresqlConfig("user", "bzip2", True, [])
        postgresql2 = PostgresqlConfig("user", "bzip2", True, [])
        self.assertEqual(postgresql1, postgresql2)
        self.assertTrue(postgresql1 == postgresql2)
        self.assertTrue(not postgresql1 < postgresql2)
        self.assertTrue(postgresql1 <= postgresql2)
        self.assertTrue(not postgresql1 > postgresql2)
        self.assertTrue(postgresql1 >= postgresql2)
        self.assertTrue(not postgresql1 != postgresql2)

    def testComparison_004(self):
        """
      Test comparison of two identical objects, all attributes non-None, list non-empty.
      """
        postgresql1 = PostgresqlConfig("user", "none", True, ["whatever",])
        postgresql2 = PostgresqlConfig("user", "none", True, ["whatever",])
        self.assertEqual(postgresql1, postgresql2)
        self.assertTrue(postgresql1 == postgresql2)
        self.assertTrue(not postgresql1 < postgresql2)
        self.assertTrue(postgresql1 <= postgresql2)
        self.assertTrue(not postgresql1 > postgresql2)
        self.assertTrue(postgresql1 >= postgresql2)
        self.assertTrue(not postgresql1 != postgresql2)

    def testComparison_005(self):
        """
      Test comparison of two differing objects, user differs (one None).
      """
        postgresql1 = PostgresqlConfig()
        postgresql2 = PostgresqlConfig(user="user")
        self.assertNotEqual(postgresql1, postgresql2)
        self.assertTrue(not postgresql1 == postgresql2)
        self.assertTrue(postgresql1 < postgresql2)
        self.assertTrue(postgresql1 <= postgresql2)
        self.assertTrue(not postgresql1 > postgresql2)
        self.assertTrue(not postgresql1 >= postgresql2)
        self.assertTrue(postgresql1 != postgresql2)

    def testComparison_006(self):
        """
      Test comparison of two differing objects, user differs.
      """
        postgresql1 = PostgresqlConfig("user1", "gzip", True, ["whatever",])
        postgresql2 = PostgresqlConfig("user2", "gzip", True, ["whatever",])
        self.assertNotEqual(postgresql1, postgresql2)
        self.assertTrue(not postgresql1 == postgresql2)
        self.assertTrue(postgresql1 < postgresql2)
        self.assertTrue(postgresql1 <= postgresql2)
        self.assertTrue(not postgresql1 > postgresql2)
        self.assertTrue(not postgresql1 >= postgresql2)
        self.assertTrue(postgresql1 != postgresql2)

    def testComparison_007(self):
        """
      Test comparison of two differing objects, compressMode differs (one None).
      """
        postgresql1 = PostgresqlConfig()
        postgresql2 = PostgresqlConfig(compressMode="gzip")
        self.assertNotEqual(postgresql1, postgresql2)
        self.assertTrue(not postgresql1 == postgresql2)
        self.assertTrue(postgresql1 < postgresql2)
        self.assertTrue(postgresql1 <= postgresql2)
        self.assertTrue(not postgresql1 > postgresql2)
        self.assertTrue(not postgresql1 >= postgresql2)
        self.assertTrue(postgresql1 != postgresql2)

    def testComparison_008(self):
        """
      Test comparison of two differing objects, compressMode differs.
      """
        postgresql1 = PostgresqlConfig("user", "bzip2", True, ["whatever",])
        postgresql2 = PostgresqlConfig("user", "gzip", True, ["whatever",])
        self.assertNotEqual(postgresql1, postgresql2)
        self.assertTrue(not postgresql1 == postgresql2)
        self.assertTrue(postgresql1 < postgresql2)
        self.assertTrue(postgresql1 <= postgresql2)
        self.assertTrue(not postgresql1 > postgresql2)
        self.assertTrue(not postgresql1 >= postgresql2)
        self.assertTrue(postgresql1 != postgresql2)

    def testComparison_009(self):
        """
      Test comparison of two differing objects, all differs (one None).
      """
        postgresql1 = PostgresqlConfig()
        postgresql2 = PostgresqlConfig(all=True)
        self.assertNotEqual(postgresql1, postgresql2)
        self.assertTrue(not postgresql1 == postgresql2)
        self.assertTrue(postgresql1 < postgresql2)
        self.assertTrue(postgresql1 <= postgresql2)
        self.assertTrue(not postgresql1 > postgresql2)
        self.assertTrue(not postgresql1 >= postgresql2)
        self.assertTrue(postgresql1 != postgresql2)

    def testComparison_010(self):
        """
      Test comparison of two differing objects, all differs.
      """
        postgresql1 = PostgresqlConfig("user", "gzip", False, ["whatever",])
        postgresql2 = PostgresqlConfig("user", "gzip", True, ["whatever",])
        self.assertNotEqual(postgresql1, postgresql2)
        self.assertTrue(not postgresql1 == postgresql2)
        self.assertTrue(postgresql1 < postgresql2)
        self.assertTrue(postgresql1 <= postgresql2)
        self.assertTrue(not postgresql1 > postgresql2)
        self.assertTrue(not postgresql1 >= postgresql2)
        self.assertTrue(postgresql1 != postgresql2)

    def testComparison_011(self):
        """
      Test comparison of two differing objects, databases differs (one None, one empty).
      """
        postgresql1 = PostgresqlConfig()
        postgresql2 = PostgresqlConfig(databases=[])
        self.assertNotEqual(postgresql1, postgresql2)
        self.assertTrue(not postgresql1 == postgresql2)
        self.assertTrue(postgresql1 < postgresql2)
        self.assertTrue(postgresql1 <= postgresql2)
        self.assertTrue(not postgresql1 > postgresql2)
        self.assertTrue(not postgresql1 >= postgresql2)
        self.assertTrue(postgresql1 != postgresql2)

    def testComparison_012(self):
        """
      Test comparison of two differing objects, databases differs (one None, one not empty).
      """
        postgresql1 = PostgresqlConfig()
        postgresql2 = PostgresqlConfig(databases=["whatever",])
        self.assertNotEqual(postgresql1, postgresql2)
        self.assertTrue(not postgresql1 == postgresql2)
        self.assertTrue(postgresql1 < postgresql2)
        self.assertTrue(postgresql1 <= postgresql2)
        self.assertTrue(not postgresql1 > postgresql2)
        self.assertTrue(not postgresql1 >= postgresql2)
        self.assertTrue(postgresql1 != postgresql2)

    def testComparison_013(self):
        """
      Test comparison of two differing objects, databases differs (one empty, one not empty).
      """
        postgresql1 = PostgresqlConfig("user", "gzip", True, [])
        postgresql2 = PostgresqlConfig("user", "gzip", True, ["whatever",])
        self.assertNotEqual(postgresql1, postgresql2)
        self.assertTrue(not postgresql1 == postgresql2)
        self.assertTrue(postgresql1 < postgresql2)
        self.assertTrue(postgresql1 <= postgresql2)
        self.assertTrue(not postgresql1 > postgresql2)
        self.assertTrue(not postgresql1 >= postgresql2)
        self.assertTrue(postgresql1 != postgresql2)

    def testComparison_014(self):
        """
      Test comparison of two differing objects, databases differs (both not empty).
      """
        postgresql1 = PostgresqlConfig("user", "gzip", True, ["whatever",])
        postgresql2 = PostgresqlConfig("user", "gzip", True, ["whatever", "bogus",])
        self.assertNotEqual(postgresql1, postgresql2)
        self.assertTrue(not postgresql1 == postgresql2)
        self.assertTrue(not postgresql1 < postgresql2)  # note: different than standard due to unsorted list
        self.assertTrue(not postgresql1 <= postgresql2)  # note: different than standard due to unsorted list
        self.assertTrue(postgresql1 > postgresql2)  # note: different than standard due to unsorted list
        self.assertTrue(postgresql1 >= postgresql2)  # note: different than standard due to unsorted list
        self.assertTrue(postgresql1 != postgresql2)


########################
# TestLocalConfig class
########################


class TestLocalConfig(unittest.TestCase):

    """Tests for the LocalConfig class."""

    ################
    # Setup methods
    ################

    @classmethod
    def setUpClass(cls):
        configureLogging()

    def setUp(self):
        try:
            self.resources = findResources(RESOURCES, DATA_DIRS)
        except Exception as e:
            self.fail(e)

    def tearDown(self):
        pass

    ##################
    # Utility methods
    ##################

    def failUnlessAssignRaises(self, exception, obj, prop, value):
        """Equivalent of :any:`failUnlessRaises`, but used for property assignments instead."""
        failUnlessAssignRaises(self, exception, obj, prop, value)

    def validateAddConfig(self, origConfig):
        """
      Validates that document dumped from ``LocalConfig.addConfig`` results in
      identical object.

      We dump a document containing just the postgresql configuration, and then make
      sure that if we push that document back into the ``LocalConfig`` object,
      that the resulting object matches the original.

      The ``self.failUnlessEqual`` method is used for the validation, so if the
      method call returns normally, everything is OK.

      Args:
         origConfig: Original configuration
      """
        (xmlDom, parentNode) = createOutputDom()
        origConfig.addConfig(xmlDom, parentNode)
        xmlData = serializeDom(xmlDom)
        newConfig = LocalConfig(xmlData=xmlData, validate=False)
        self.assertEqual(origConfig, newConfig)

    ############################
    # Test __repr__ and __str__
    ############################

    def testStringFuncs_001(self):
        """
      Just make sure that the string functions don't have errors (i.e. bad variable names).
      """
        obj = LocalConfig()
        obj.__repr__()
        obj.__str__()

    #####################################################
    # Test basic constructor and attribute functionality
    #####################################################

    def testConstructor_001(self):
        """
      Test empty constructor, validate=False.
      """
        config = LocalConfig(validate=False)
        self.assertEqual(None, config.postgresql)

    def testConstructor_002(self):
        """
      Test empty constructor, validate=True.
      """
        config = LocalConfig(validate=True)
        self.assertEqual(None, config.postgresql)

    def testConstructor_003(self):
        """
      Test with empty config document as both data and file, validate=False.
      """
        path = self.resources["postgresql.conf.1"]
        with open(path) as f:
            contents = f.read()
        self.assertRaises(ValueError, LocalConfig, xmlData=contents, xmlPath=path, validate=False)

    def testConstructor_004(self):
        """
      Test assignment of postgresql attribute, None value.
      """
        config = LocalConfig()
        config.postgresql = None
        self.assertEqual(None, config.postgresql)

    def testConstructor_005(self):
        """
      Test assignment of postgresql attribute, valid value.
      """
        config = LocalConfig()
        config.postgresql = PostgresqlConfig()
        self.assertEqual(PostgresqlConfig(), config.postgresql)

    def testConstructor_006(self):
        """
      Test assignment of postgresql attribute, invalid value (not PostgresqlConfig).
      """
        config = LocalConfig()
        self.failUnlessAssignRaises(ValueError, config, "postgresql", "STRING!")

    ############################
    # Test comparison operators
    ############################

    def testComparison_001(self):
        """
      Test comparison of two identical objects, all attributes None.
      """
        config1 = LocalConfig()
        config2 = LocalConfig()
        self.assertEqual(config1, config2)
        self.assertTrue(config1 == config2)
        self.assertTrue(not config1 < config2)
        self.assertTrue(config1 <= config2)
        self.assertTrue(not config1 > config2)
        self.assertTrue(config1 >= config2)
        self.assertTrue(not config1 != config2)

    def testComparison_002(self):
        """
      Test comparison of two identical objects, all attributes non-None.
      """
        config1 = LocalConfig()
        config1.postgresql = PostgresqlConfig()

        config2 = LocalConfig()
        config2.postgresql = PostgresqlConfig()

        self.assertEqual(config1, config2)
        self.assertTrue(config1 == config2)
        self.assertTrue(not config1 < config2)
        self.assertTrue(config1 <= config2)
        self.assertTrue(not config1 > config2)
        self.assertTrue(config1 >= config2)
        self.assertTrue(not config1 != config2)

    def testComparison_003(self):
        """
      Test comparison of two differing objects, postgresql differs (one None).
      """
        config1 = LocalConfig()
        config2 = LocalConfig()
        config2.postgresql = PostgresqlConfig()
        self.assertNotEqual(config1, config2)
        self.assertTrue(not config1 == config2)
        self.assertTrue(config1 < config2)
        self.assertTrue(config1 <= config2)
        self.assertTrue(not config1 > config2)
        self.assertTrue(not config1 >= config2)
        self.assertTrue(config1 != config2)

    def testComparison_004(self):
        """
      Test comparison of two differing objects, postgresql differs.
      """
        config1 = LocalConfig()
        config1.postgresql = PostgresqlConfig(user="one")

        config2 = LocalConfig()
        config2.postgresql = PostgresqlConfig(user="two")

        self.assertNotEqual(config1, config2)
        self.assertTrue(not config1 == config2)
        self.assertTrue(config1 < config2)
        self.assertTrue(config1 <= config2)
        self.assertTrue(not config1 > config2)
        self.assertTrue(not config1 >= config2)
        self.assertTrue(config1 != config2)

    ######################
    # Test validate logic
    ######################

    def testValidate_001(self):
        """
      Test validate on a None postgresql section.
      """
        config = LocalConfig()
        config.postgresql = None
        self.assertRaises(ValueError, config.validate)

    def testValidate_002(self):
        """
      Test validate on an empty postgresql section.
      """
        config = LocalConfig()
        config.postgresql = PostgresqlConfig()
        self.assertRaises(ValueError, config.validate)

    def testValidate_003(self):
        """
      Test validate on a non-empty postgresql section, all=True, databases=None.
      """
        config = LocalConfig()
        config.postgresql = PostgresqlConfig("user", "gzip", True, None)
        config.validate()

    def testValidate_004(self):
        """
      Test validate on a non-empty postgresql section, all=True, empty databases.
      """
        config = LocalConfig()
        config.postgresql = PostgresqlConfig("user", "none", True, [])
        config.validate()

    def testValidate_005(self):
        """
      Test validate on a non-empty postgresql section, all=True, non-empty databases.
      """
        config = LocalConfig()
        config.postgresql = PostgresqlConfig("user", "bzip2", True, ["whatever",])
        self.assertRaises(ValueError, config.validate)

    def testValidate_006(self):
        """
      Test validate on a non-empty postgresql section, all=False, databases=None.
      """
        config = LocalConfig()
        config.postgresql = PostgresqlConfig("user", "gzip", False, None)
        self.assertRaises(ValueError, config.validate)

    def testValidate_007(self):
        """
      Test validate on a non-empty postgresql section, all=False, empty databases.
      """
        config = LocalConfig()
        config.postgresql = PostgresqlConfig("user", "bzip2", False, [])
        self.assertRaises(ValueError, config.validate)

    def testValidate_008(self):
        """
      Test validate on a non-empty postgresql section, all=False, non-empty databases.
      """
        config = LocalConfig()
        config.postgresql = PostgresqlConfig("user", "gzip", False, ["whatever",])
        config.validate()

    def testValidate_009(self):
        """
      Test validate on a non-empty postgresql section, with user=None.
      """
        config = LocalConfig()
        config.postgresql = PostgresqlConfig(None, "gzip", True, None)
        config.validate()

    ############################
    # Test parsing of documents
    ############################

    def testParse_001(self):
        """
      Parse empty config document.
      """
        path = self.resources["postgresql.conf.1"]
        with open(path) as f:
            contents = f.read()
        self.assertRaises(ValueError, LocalConfig, xmlPath=path, validate=True)
        self.assertRaises(ValueError, LocalConfig, xmlData=contents, validate=True)
        config = LocalConfig(xmlPath=path, validate=False)
        self.assertEqual(None, config.postgresql)
        config = LocalConfig(xmlData=contents, validate=False)
        self.assertEqual(None, config.postgresql)

    def testParse_003(self):
        """
      Parse config document containing only a postgresql section, no databases, all=True.
      """
        path = self.resources["postgresql.conf.2"]
        with open(path) as f:
            contents = f.read()
        config = LocalConfig(xmlPath=path, validate=False)
        self.assertNotEqual(None, config.postgresql)
        self.assertEqual("user", config.postgresql.user)
        self.assertEqual("none", config.postgresql.compressMode)
        self.assertEqual(True, config.postgresql.all)
        self.assertEqual(None, config.postgresql.databases)
        config = LocalConfig(xmlData=contents, validate=False)
        self.assertEqual("user", config.postgresql.user)
        self.assertEqual("none", config.postgresql.compressMode)
        self.assertEqual(True, config.postgresql.all)
        self.assertEqual(None, config.postgresql.databases)

    def testParse_004(self):
        """
      Parse config document containing only a postgresql section, single database, all=False.
      """
        path = self.resources["postgresql.conf.3"]
        with open(path) as f:
            contents = f.read()
        config = LocalConfig(xmlPath=path, validate=False)
        self.assertNotEqual(None, config.postgresql)
        self.assertEqual("user", config.postgresql.user)
        self.assertEqual("gzip", config.postgresql.compressMode)
        self.assertEqual(False, config.postgresql.all)
        self.assertEqual(["database",], config.postgresql.databases)
        config = LocalConfig(xmlData=contents, validate=False)
        self.assertNotEqual(None, config.postgresql)
        self.assertEqual("user", config.postgresql.user)
        self.assertEqual("gzip", config.postgresql.compressMode)
        self.assertEqual(False, config.postgresql.all)
        self.assertEqual(["database",], config.postgresql.databases)

    def testParse_005(self):
        """
      Parse config document containing only a postgresql section, multiple databases, all=False.
      """
        path = self.resources["postgresql.conf.4"]
        with open(path) as f:
            contents = f.read()
        config = LocalConfig(xmlPath=path, validate=False)
        self.assertNotEqual(None, config.postgresql)
        self.assertEqual("user", config.postgresql.user)
        self.assertEqual("bzip2", config.postgresql.compressMode)
        self.assertEqual(False, config.postgresql.all)
        self.assertEqual(["database1", "database2",], config.postgresql.databases)
        config = LocalConfig(xmlData=contents, validate=False)
        self.assertNotEqual(None, config.postgresql)
        self.assertEqual("user", config.postgresql.user)
        self.assertEqual("bzip2", config.postgresql.compressMode)
        self.assertEqual(False, config.postgresql.all)
        self.assertEqual(["database1", "database2",], config.postgresql.databases)

    def testParse_006(self):
        """
      Parse config document containing only a postgresql section, no user, multiple databases, all=False.
      """
        path = self.resources["postgresql.conf.5"]
        with open(path) as f:
            contents = f.read()
        config = LocalConfig(xmlPath=path, validate=False)
        self.assertNotEqual(None, config.postgresql)
        self.assertEqual(None, config.postgresql.user)
        self.assertEqual("bzip2", config.postgresql.compressMode)
        self.assertEqual(False, config.postgresql.all)
        self.assertEqual(["database1", "database2",], config.postgresql.databases)
        config = LocalConfig(xmlData=contents, validate=False)
        self.assertNotEqual(None, config.postgresql)
        self.assertEqual(None, config.postgresql.user)
        self.assertEqual("bzip2", config.postgresql.compressMode)
        self.assertEqual(False, config.postgresql.all)
        self.assertEqual(["database1", "database2",], config.postgresql.databases)

    ###################
    # Test addConfig()
    ###################

    def testAddConfig_001(self):
        """
      Test with empty config document
      """
        config = LocalConfig()
        self.validateAddConfig(config)

    def testAddConfig_003(self):
        """
      Test with no databases, all other values filled in, all=True.
      """
        config = LocalConfig()
        config.postgresql = PostgresqlConfig("user", "none", True, None)
        self.validateAddConfig(config)

    def testAddConfig_004(self):
        """
      Test with no databases, all other values filled in, all=False.
      """
        config = LocalConfig()
        config.postgresql = PostgresqlConfig("user", "gzip", False, None)
        self.validateAddConfig(config)

    def testAddConfig_005(self):
        """
      Test with single database, all other values filled in, all=True.
      """
        config = LocalConfig()
        config.postgresql = PostgresqlConfig("user", "bzip2", True, ["database",])
        self.validateAddConfig(config)

    def testAddConfig_006(self):
        """
      Test with single database, all other values filled in, all=False.
      """
        config = LocalConfig()
        config.postgresql = PostgresqlConfig("user", "none", False, ["database",])
        self.validateAddConfig(config)

    def testAddConfig_007(self):
        """
      Test with multiple databases, all other values filled in, all=True.
      """
        config = LocalConfig()
        config.postgresql = PostgresqlConfig("user", "bzip2", True, ["database1", "database2",])
        self.validateAddConfig(config)

    def testAddConfig_008(self):
        """
      Test with multiple databases, all other values filled in, all=False.
      """
        config = LocalConfig()
        config.postgresql = PostgresqlConfig("user", "gzip", True, ["database1", "database2",])
        self.validateAddConfig(config)

    def testAddConfig_009(self):
        """
      Test with multiple databases, user=None but all other values filled in, all=False.
      """
        config = LocalConfig()
        config.postgresql = PostgresqlConfig(None, "gzip", True, ["database1", "database2",])
        self.validateAddConfig(config)



# -*- coding: iso-8859-1 -*-
# vim: set ft=python ts=4 sw=4 expandtab:
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
#              C E D A R
#          S O L U T I O N S       "Software done right."
#           S O F T W A R E
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
# Copyright (c) 2007,2010,2015 Kenneth J. Pronovici.
# All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License,
# Version 2, as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# Copies of the GNU General Public License are available from
# the Free Software Foundation website, http://www.gnu.org/.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
# Author   : Kenneth J. Pronovici <pronovic@ieee.org>
# Language : Python 3 (>= 3.7)
# Project  : Official Cedar Backup Extensions
# Purpose  : Provides an extension to encrypt staging directories.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

########################################################################
# Module documentation
########################################################################

"""
Provides an extension to encrypt staging directories.

When this extension is executed, all backed-up files in the configured Cedar
Backup staging directory will be encrypted using gpg.  Any directory which has
already been encrypted (as indicated by the ``cback.encrypt`` file) will be
ignored.

This extension requires a new configuration section <encrypt> and is intended
to be run immediately after the standard stage action or immediately before the
standard store action.  Aside from its own configuration, it requires the
options and staging configuration sections in the standard Cedar Backup
configuration file.

:author: Kenneth J. Pronovici <pronovic@ieee.org>
"""

########################################################################
# Imported modules
########################################################################

import logging
import os
from functools import total_ordering

from CedarBackup3.actions.util import findDailyDirs, getBackupFiles, writeIndicatorFile
from CedarBackup3.util import changeOwnership, executeCommand, resolveCommand
from CedarBackup3.xmlutil import addContainerNode, addStringNode, createInputDom, readFirstChild, readString

########################################################################
# Module-wide constants and variables
########################################################################

logger = logging.getLogger("CedarBackup3.log.extend.encrypt")

GPG_COMMAND = [
    "gpg",
]
VALID_ENCRYPT_MODES = [
    "gpg",
]
ENCRYPT_INDICATOR = "cback.encrypt"


########################################################################
# EncryptConfig class definition
########################################################################


@total_ordering
class EncryptConfig(object):

    """
   Class representing encrypt configuration.

   Encrypt configuration is used for encrypting staging directories.

   The following restrictions exist on data in this class:

      - The encrypt mode must be one of the values in ``VALID_ENCRYPT_MODES``
      - The encrypt target value must be a non-empty string

   """

    def __init__(self, encryptMode=None, encryptTarget=None):
        """
      Constructor for the ``EncryptConfig`` class.

      Args:
         encryptMode: Encryption mode
         encryptTarget: Encryption target (for instance, GPG recipient)

      Raises:
         ValueError: If one of the values is invalid
      """
        self._encryptMode = None
        self._encryptTarget = None
        self.encryptMode = encryptMode
        self.encryptTarget = encryptTarget

    def __repr__(self):
        """
      Official string representation for class instance.
      """
        return "EncryptConfig(%s, %s)" % (self.encryptMode, self.encryptTarget)

    def __str__(self):
        """
      Informal string representation for class instance.
      """
        return self.__repr__()

    def __eq__(self, other):
        """Equals operator, iplemented in terms of original Python 2 compare operator."""
        return self.__cmp__(other) == 0

    def __lt__(self, other):
        """Less-than operator, iplemented in terms of original Python 2 compare operator."""
        return self.__cmp__(other) < 0

    def __gt__(self, other):
        """Greater-than operator, iplemented in terms of original Python 2 compare operator."""
        return self.__cmp__(other) > 0

    def __cmp__(self, other):
        """
      Original Python 2 comparison operator.
      Lists within this class are "unordered" for equality comparisons.
      Args:
         other: Other object to compare to
      Returns:
          -1/0/1 depending on whether self is ``<``, ``=`` or ``>`` other
      """
        if other is None:
            return 1
        if self.encryptMode != other.encryptMode:
            if str(self.encryptMode or "") < str(other.encryptMode or ""):
                return -1
            else:
                return 1
        if self.encryptTarget != other.encryptTarget:
            if str(self.encryptTarget or "") < str(other.encryptTarget or ""):
                return -1
            else:
                return 1
        return 0

    def _setEncryptMode(self, value):
        """
      Property target used to set the encrypt mode.
      If not ``None``, the mode must be one of the values in ``VALID_ENCRYPT_MODES``.
      Raises:
         ValueError: If the value is not valid
      """
        if value is not None:
            if value not in VALID_ENCRYPT_MODES:
                raise ValueError("Encrypt mode must be one of %s." % VALID_ENCRYPT_MODES)
        self._encryptMode = value

    def _getEncryptMode(self):
        """
      Property target used to get the encrypt mode.
      """
        return self._encryptMode

    def _setEncryptTarget(self, value):
        """
      Property target used to set the encrypt target.
      """
        if value is not None:
            if len(value) < 1:
                raise ValueError("Encrypt target must be non-empty string.")
        self._encryptTarget = value

    def _getEncryptTarget(self):
        """
      Property target used to get the encrypt target.
      """
        return self._encryptTarget

    encryptMode = property(_getEncryptMode, _setEncryptMode, None, doc="Encrypt mode.")
    encryptTarget = property(_getEncryptTarget, _setEncryptTarget, None, doc="Encrypt target (i.e. GPG recipient).")


########################################################################
# LocalConfig class definition
########################################################################


@total_ordering
class LocalConfig(object):

    """
   Class representing this extension's configuration document.

   This is not a general-purpose configuration object like the main Cedar
   Backup configuration object.  Instead, it just knows how to parse and emit
   encrypt-specific configuration values.  Third parties who need to read and
   write configuration related to this extension should access it through the
   constructor, ``validate`` and ``addConfig`` methods.

   *Note:* Lists within this class are "unordered" for equality comparisons.

   """

    def __init__(self, xmlData=None, xmlPath=None, validate=True):
        """
      Initializes a configuration object.

      If you initialize the object without passing either ``xmlData`` or
      ``xmlPath`` then configuration will be empty and will be invalid until it
      is filled in properly.

      No reference to the original XML data or original path is saved off by
      this class.  Once the data has been parsed (successfully or not) this
      original information is discarded.

      Unless the ``validate`` argument is ``False``, the :any:`LocalConfig.validate`
      method will be called (with its default arguments) against configuration
      after successfully parsing any passed-in XML.  Keep in mind that even if
      ``validate`` is ``False``, it might not be possible to parse the passed-in
      XML document if lower-level validations fail.

      *Note:* It is strongly suggested that the ``validate`` option always be set
      to ``True`` (the default) unless there is a specific need to read in
      invalid configuration from disk.

      Args:
         xmlData (String data): XML data representing configuration
         xmlPath (Absolute path to a file on disk): Path to an XML file on disk
         validate (Boolean true/false): Validate the document after parsing it
      Raises:
         ValueError: If both ``xmlData`` and ``xmlPath`` are passed-in
         ValueError: If the XML data in ``xmlData`` or ``xmlPath`` cannot be parsed
         ValueError: If the parsed configuration document is not valid
      """
        self._encrypt = None
        self.encrypt = None
        if xmlData is not None and xmlPath is not None:
            raise ValueError("Use either xmlData or xmlPath, but not both.")
        if xmlData is not None:
            self._parseXmlData(xmlData)
            if validate:
                self.validate()
        elif xmlPath is not None:
            with open(xmlPath) as f:
                xmlData = f.read()
            self._parseXmlData(xmlData)
            if validate:
                self.validate()

    def __repr__(self):
        """
      Official string representation for class instance.
      """
        return "LocalConfig(%s)" % (self.encrypt)

    def __str__(self):
        """
      Informal string representation for class instance.
      """
        return self.__repr__()

    def __eq__(self, other):
        """Equals operator, iplemented in terms of original Python 2 compare operator."""
        return self.__cmp__(other) == 0

    def __lt__(self, other):
        """Less-than operator, iplemented in terms of original Python 2 compare operator."""
        return self.__cmp__(other) < 0

    def __gt__(self, other):
        """Greater-than operator, iplemented in terms of original Python 2 compare operator."""
        return self.__cmp__(other) > 0

    def __cmp__(self, other):
        """
      Original Python 2 comparison operator.
      Lists within this class are "unordered" for equality comparisons.
      Args:
         other: Other object to compare to
      Returns:
          -1/0/1 depending on whether self is ``<``, ``=`` or ``>`` other
      """
        if other is None:
            return 1
        if self.encrypt != other.encrypt:
            if self.encrypt < other.encrypt:
                return -1
            else:
                return 1
        return 0

    def _setEncrypt(self, value):
        """
      Property target used to set the encrypt configuration value.
      If not ``None``, the value must be a ``EncryptConfig`` object.
      Raises:
         ValueError: If the value is not a ``EncryptConfig``
      """
        if value is None:
            self._encrypt = None
        else:
            if not isinstance(value, EncryptConfig):
                raise ValueError("Value must be a ``EncryptConfig`` object.")
            self._encrypt = value

    def _getEncrypt(self):
        """
      Property target used to get the encrypt configuration value.
      """
        return self._encrypt

    encrypt = property(_getEncrypt, _setEncrypt, None, "Encrypt configuration in terms of a ``EncryptConfig`` object.")

    def validate(self):
        """
      Validates configuration represented by the object.

      Encrypt configuration must be filled in.  Within that, both the encrypt
      mode and encrypt target must be filled in.

      Raises:
         ValueError: If one of the validations fails
      """
        if self.encrypt is None:
            raise ValueError("Encrypt section is required.")
        if self.encrypt.encryptMode is None:
            raise ValueError("Encrypt mode must be set.")
        if self.encrypt.encryptTarget is None:
            raise ValueError("Encrypt target must be set.")

    def addConfig(self, xmlDom, parentNode):
        """
      Adds an <encrypt> configuration section as the next child of a parent.

      Third parties should use this function to write configuration related to
      this extension.

      We add the following fields to the document::

         encryptMode    //cb_config/encrypt/encrypt_mode
         encryptTarget  //cb_config/encrypt/encrypt_target

      Args:
         xmlDom: DOM tree as from ``impl.createDocument()``
         parentNode: Parent that the section should be appended to
      """
        if self.encrypt is not None:
            sectionNode = addContainerNode(xmlDom, parentNode, "encrypt")
            addStringNode(xmlDom, sectionNode, "encrypt_mode", self.encrypt.encryptMode)
            addStringNode(xmlDom, sectionNode, "encrypt_target", self.encrypt.encryptTarget)

    def _parseXmlData(self, xmlData):
        """
      Internal method to parse an XML string into the object.

      This method parses the XML document into a DOM tree (``xmlDom``) and then
      calls a static method to parse the encrypt configuration section.

      Args:
         xmlData (String data): XML data to be parsed
      Raises:
         ValueError: If the XML cannot be successfully parsed
      """
        (xmlDom, parentNode) = createInputDom(xmlData)
        self._encrypt = LocalConfig._parseEncrypt(parentNode)

    @staticmethod
    def _parseEncrypt(parent):
        """
      Parses an encrypt configuration section.

      We read the following individual fields::

         encryptMode    //cb_config/encrypt/encrypt_mode
         encryptTarget  //cb_config/encrypt/encrypt_target

      Args:
         parent: Parent node to search beneath

      Returns:
          ``EncryptConfig`` object or ``None`` if the section does not exist
      Raises:
         ValueError: If some filled-in value is invalid
      """
        encrypt = None
        section = readFirstChild(parent, "encrypt")
        if section is not None:
            encrypt = EncryptConfig()
            encrypt.encryptMode = readString(section, "encrypt_mode")
            encrypt.encryptTarget = readString(section, "encrypt_target")
        return encrypt


########################################################################
# Public functions
########################################################################

###########################
# executeAction() function
###########################

# pylint: disable=W0613
def executeAction(configPath, options, config):
    """
   Executes the encrypt backup action.

   Args:
      configPath (String representing a path on disk): Path to configuration file on disk
      options (Options object): Program command-line options
      config (Config object): Program configuration
   Raises:
      ValueError: Under many generic error conditions
      IOError: If there are I/O problems reading or writing files
   """
    logger.debug("Executing encrypt extended action.")
    if config.options is None or config.stage is None:
        raise ValueError("Cedar Backup configuration is not properly filled in.")
    local = LocalConfig(xmlPath=configPath)
    if local.encrypt.encryptMode not in ["gpg"]:
        raise ValueError("Unknown encrypt mode [%s]" % local.encrypt.encryptMode)
    if local.encrypt.encryptMode == "gpg":
        _confirmGpgRecipient(local.encrypt.encryptTarget)
    dailyDirs = findDailyDirs(config.stage.targetDir, ENCRYPT_INDICATOR)
    for dailyDir in dailyDirs:
        _encryptDailyDir(
            dailyDir, local.encrypt.encryptMode, local.encrypt.encryptTarget, config.options.backupUser, config.options.backupGroup
        )
        writeIndicatorFile(dailyDir, ENCRYPT_INDICATOR, config.options.backupUser, config.options.backupGroup)
    logger.info("Executed the encrypt extended action successfully.")


##############################
# _encryptDailyDir() function
##############################


def _encryptDailyDir(dailyDir, encryptMode, encryptTarget, backupUser, backupGroup):
    """
   Encrypts the contents of a daily staging directory.

   Indicator files are ignored.  All other files are encrypted.  The only valid
   encrypt mode is ``"gpg"``.

   Args:
      dailyDir: Daily directory to encrypt
      encryptMode: Encryption mode (only "gpg" is allowed)
      encryptTarget: Encryption target (GPG recipient for "gpg" mode)
      backupUser: User that target files should be owned by
      backupGroup: Group that target files should be owned by

   Raises:
      ValueError: If the encrypt mode is not supported
      ValueError: If the daily staging directory does not exist
   """
    logger.debug("Begin encrypting contents of [%s].", dailyDir)
    fileList = getBackupFiles(dailyDir)  # ignores indicator files
    for path in fileList:
        _encryptFile(path, encryptMode, encryptTarget, backupUser, backupGroup, removeSource=True)
    logger.debug("Completed encrypting contents of [%s].", dailyDir)


##########################
# _encryptFile() function
##########################


def _encryptFile(sourcePath, encryptMode, encryptTarget, backupUser, backupGroup, removeSource=False):
    """
   Encrypts the source file using the indicated mode.

   The encrypted file will be owned by the indicated backup user and group.  If
   ``removeSource`` is ``True``, then the source file will be removed after it is
   successfully encrypted.

   Currently, only the ``"gpg"`` encrypt mode is supported.

   Args:
      sourcePath: Absolute path of the source file to encrypt
      encryptMode: Encryption mode (only "gpg" is allowed)
      encryptTarget: Encryption target (GPG recipient)
      backupUser: User that target files should be owned by
      backupGroup: Group that target files should be owned by
      removeSource: Indicates whether to remove the source file

   Returns:
       Path to the newly-created encrypted file

   Raises:
      ValueError: If an invalid encrypt mode is passed in
      IOError: If there is a problem accessing, encrypting or removing the source file
   """
    if not os.path.exists(sourcePath):
        raise ValueError("Source path [%s] does not exist." % sourcePath)
    if encryptMode == "gpg":
        encryptedPath = _encryptFileWithGpg(sourcePath, recipient=encryptTarget)
    else:
        raise ValueError("Unknown encrypt mode [%s]" % encryptMode)
    changeOwnership(encryptedPath, backupUser, backupGroup)
    if removeSource:
        if os.path.exists(sourcePath):
            try:
                os.remove(sourcePath)
                logger.debug("Completed removing old file [%s].", sourcePath)
            except:
                raise IOError("Failed to remove file [%s] after encrypting it." % (sourcePath))
    return encryptedPath


#################################
# _encryptFileWithGpg() function
#################################


def _encryptFileWithGpg(sourcePath, recipient):
    """
   Encrypts the indicated source file using GPG.

   The encrypted file will be in GPG's binary output format and will have the
   same name as the source file plus a ``".gpg"`` extension.  The source file
   will not be modified or removed by this function call.

   Args:
      sourcePath: Absolute path of file to be encrypted
      recipient: Recipient name to be passed to GPG's ``"-r"`` option

   Returns:
       Path to the newly-created encrypted file

   Raises:
      IOError: If there is a problem encrypting the file
   """
    encryptedPath = "%s.gpg" % sourcePath
    command = resolveCommand(GPG_COMMAND)
    args = ["--batch", "--yes", "-e", "-r", recipient, "-o", encryptedPath, sourcePath]
    result = executeCommand(command, args)[0]
    if result != 0:
        raise IOError("Error [%d] calling gpg to encrypt [%s]." % (result, sourcePath))
    if not os.path.exists(encryptedPath):
        raise IOError("After call to [%s], encrypted file [%s] does not exist." % (command, encryptedPath))
    logger.debug("Completed encrypting file [%s] to [%s].", sourcePath, encryptedPath)
    return encryptedPath


#################################
# _confirmGpgRecpient() function
#################################


def _confirmGpgRecipient(recipient):
    """
   Confirms that a recipient's public key is known to GPG.
   Throws an exception if there is a problem, or returns normally otherwise.
   Args:
      recipient: Recipient name
   Raises:
      IOError: If the recipient's public key is not known to GPG
   """
    command = resolveCommand(GPG_COMMAND)
    args = ["--batch", "-k", recipient]  # should use --with-colons if the output will be parsed
    result = executeCommand(command, args)[0]
    if result != 0:
        raise IOError("GPG unable to find public key for [%s]." % recipient)



# -*- coding: iso-8859-1 -*-
# vim: set ft=python ts=4 sw=4 expandtab:
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
#              C E D A R
#          S O L U T I O N S       "Software done right."
#           S O F T W A R E
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
# Copyright (c) 2004-2007,2010,2015 Kenneth J. Pronovici.
# All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License,
# Version 2, as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# Copies of the GNU General Public License are available from
# the Free Software Foundation website, http://www.gnu.org/.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
# Author   : Kenneth J. Pronovici <pronovic@ieee.org>
# Language : Python 3 (>= 3.7)
# Project  : Cedar Backup, release 3
# Purpose  : Provides utilities related to image writers.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

########################################################################
# Module documentation
########################################################################

"""
Provides utilities related to image writers.
:author: Kenneth J. Pronovici <pronovic@ieee.org>
"""


########################################################################
# Imported modules
########################################################################

import logging
import os
import re

from CedarBackup3.util import UNIT_BYTES, UNIT_SECTORS, convertSize, encodePath, executeCommand, resolveCommand

########################################################################
# Module-wide constants and variables
########################################################################

logger = logging.getLogger("CedarBackup3.log.writers.util")

MKISOFS_COMMAND = ["mkisofs"]
VOLNAME_COMMAND = ["volname"]


########################################################################
# Functions used to portably validate certain kinds of values
########################################################################

############################
# validateDevice() function
############################


def validateDevice(device, unittest=False):
    """
   Validates a configured device.
   The device must be an absolute path, must exist, and must be writable.
   The unittest flag turns off validation of the device on disk.
   Args:
      device: Filesystem device path
      unittest: Indicates whether we're unit testing
   Returns:
       Device as a string, for instance ``"/dev/cdrw"``
   Raises:
      ValueError: If the device value is invalid
      ValueError: If some path cannot be encoded properly
   """
    if device is None:
        raise ValueError("Device must be filled in.")
    device = encodePath(device)
    if not os.path.isabs(device):
        raise ValueError("Backup device must be an absolute path.")
    if not unittest and not os.path.exists(device):
        raise ValueError("Backup device must exist on disk.")
    if not unittest and not os.access(device, os.W_OK):
        raise ValueError("Backup device is not writable by the current user.")
    return device


############################
# validateScsiId() function
############################


def validateScsiId(scsiId):
    """
   Validates a SCSI id string.
   SCSI id must be a string in the form ``[<method>:]scsibus,target,lun``.
   For Mac OS X (Darwin), we also accept the form ``IO.*Services[/N]``.
   *Note:* For consistency, if ``None`` is passed in, ``None`` will be returned.
   Args:
      scsiId: SCSI id for the device
   Returns:
       SCSI id as a string, for instance ``"ATA:1,0,0"``
   Raises:
      ValueError: If the SCSI id string is invalid
   """
    if scsiId is not None:
        pattern = re.compile(r"^\s*(.*:)?\s*[0-9][0-9]*\s*,\s*[0-9][0-9]*\s*,\s*[0-9][0-9]*\s*$")
        if not pattern.search(scsiId):
            pattern = re.compile(r"^\s*IO.*Services(/[0-9][0-9]*)?\s*$")
            if not pattern.search(scsiId):
                raise ValueError("SCSI id is not in a valid form.")
    return scsiId


################################
# validateDriveSpeed() function
################################


def validateDriveSpeed(driveSpeed):
    """
   Validates a drive speed value.
   Drive speed must be an integer which is >= 1.
   *Note:* For consistency, if ``None`` is passed in, ``None`` will be returned.
   Args:
      driveSpeed: Speed at which the drive writes
   Returns:
       Drive speed as an integer
   Raises:
      ValueError: If the drive speed value is invalid
   """
    if driveSpeed is None:
        return None
    try:
        intSpeed = int(driveSpeed)
    except TypeError:
        raise ValueError("Drive speed must be an integer >= 1.")
    if intSpeed < 1:
        raise ValueError("Drive speed must an integer >= 1.")
    return intSpeed


########################################################################
# General writer-related utility functions
########################################################################

############################
# readMediaLabel() function
############################


def readMediaLabel(devicePath):
    """
   Reads the media label (volume name) from the indicated device.
   The volume name is read using the ``volname`` command.
   Args:
      devicePath: Device path to read from
   Returns:
       Media label as a string, or None if there is no name or it could not be read
   """
    args = [devicePath]
    command = resolveCommand(VOLNAME_COMMAND)
    (result, output) = executeCommand(command, args, returnOutput=True, ignoreStderr=True)
    if result != 0:
        return None
    if output is None or len(output) < 1:
        return None
    return output[0].rstrip()


########################################################################
# IsoImage class definition
########################################################################


class IsoImage(object):

    ######################
    # Class documentation
    ######################

    """
   Represents an ISO filesystem image.

   **Summary**

   This object represents an ISO 9660 filesystem image.  It is implemented
   in terms of the ``mkisofs`` program, which has been ported to many
   operating systems and platforms.  A "sensible subset" of the ``mkisofs``
   functionality is made available through the public interface, allowing
   callers to set a variety of basic options such as publisher id,
   application id, etc. as well as specify exactly which files and
   directories they want included in their image.

   By default, the image is created using the Rock Ridge protocol (using the
   ``-r`` option to ``mkisofs``) because Rock Ridge discs are generally more
   useful on UN*X filesystems than standard ISO 9660 images.  However,
   callers can fall back to the default ``mkisofs`` functionality by setting
   the ``useRockRidge`` instance variable to ``False``.  Note, however, that
   this option is not well-tested.

   **Where Files and Directories are Placed in the Image**

   Although this class is implemented in terms of the ``mkisofs`` program,
   its standard "image contents" semantics are slightly different than the original
   ``mkisofs`` semantics.  The difference is that files and directories are
   added to the image with some additional information about their source
   directory kept intact.

   As an example, suppose you add the file ``/etc/profile`` to your image and
   you do not configure a graft point.  The file ``/profile`` will be created
   in the image.  The behavior for directories is similar.  For instance,
   suppose that you add ``/etc/X11`` to the image and do not configure a
   graft point.  In this case, the directory ``/X11`` will be created in the
   image, even if the original ``/etc/X11`` directory is empty.  I{This
   behavior differs from the standard ``mkisofs`` behavior!}

   If a graft point is configured, it will be used to modify the point at
   which a file or directory is added into an image.  Using the examples
   from above, let's assume you set a graft point of ``base`` when adding
   ``/etc/profile`` and ``/etc/X11`` to your image.  In this case, the file
   ``/base/profile`` and the directory ``/base/X11`` would be added to the
   image.

   I feel that this behavior is more consistent than the original ``mkisofs``
   behavior.  However, to be fair, it is not quite as flexible, and some
   users might not like it.  For this reason, the ``contentsOnly`` parameter
   to the :any:`addEntry` method can be used to revert to the original behavior
   if desired.

   """

    ##############
    # Constructor
    ##############

    def __init__(self, device=None, boundaries=None, graftPoint=None):
        """
      Initializes an empty ISO image object.

      Only the most commonly-used configuration items can be set using this
      constructor.  If you have a need to change the others, do so immediately
      after creating your object.

      The device and boundaries values are both required in order to write
      multisession discs.  If either is missing or ``None``, a multisession disc
      will not be written.  The boundaries tuple is in terms of ISO sectors, as
      built by an image writer class and returned in a ``writer.MediaCapacity``
      object.

      The boundaries parameter is a tuple of ``(last_sess_start, next_sess_start)``,
      as returned from ``cdrecord -msinfo``.

      Args:
         device (Either be a filesystem path or a SCSI address): Name of the device that the image will be written to
         boundaries: Session boundaries as required by ``mkisofs``, or ``None``
         graftPoint (String representing a graft point path (see :any:`addEntry`)): Default graft point for this page
      """
        self._device = None
        self._boundaries = None
        self._graftPoint = None
        self._useRockRidge = True
        self._applicationId = None
        self._biblioFile = None
        self._publisherId = None
        self._preparerId = None
        self._volumeId = None
        self.entries = {}
        self.device = device
        self.boundaries = boundaries
        self.graftPoint = graftPoint
        self.useRockRidge = True
        self.applicationId = None
        self.biblioFile = None
        self.publisherId = None
        self.preparerId = None
        self.volumeId = None
        logger.debug("Created new ISO image object.")

    #############
    # Properties
    #############

    def _setDevice(self, value):
        """
      Property target used to set the device value.
      If not ``None``, the value can be either an absolute path or a SCSI id.
      Raises:
         ValueError: If the value is not valid
      """
        try:
            if value is None:
                self._device = None
            else:
                if os.path.isabs(value):
                    self._device = value
                else:
                    self._device = validateScsiId(value)
        except ValueError:
            raise ValueError("Device must either be an absolute path or a valid SCSI id.")

    def _getDevice(self):
        """
      Property target used to get the device value.
      """
        return self._device

    def _setBoundaries(self, value):
        """
      Property target used to set the boundaries tuple.
      If not ``None``, the value must be a tuple of two integers.
      Raises:
         ValueError: If the tuple values are not integers
         IndexError: If the tuple does not contain enough elements
      """
        if value is None:
            self._boundaries = None
        else:
            self._boundaries = (int(value[0]), int(value[1]))

    def _getBoundaries(self):
        """
      Property target used to get the boundaries value.
      """
        return self._boundaries

    def _setGraftPoint(self, value):
        """
      Property target used to set the graft point.
      The value must be a non-empty string if it is not ``None``.
      Raises:
         ValueError: If the value is an empty string
      """
        if value is not None:
            if len(value) < 1:
                raise ValueError("The graft point must be a non-empty string.")
        self._graftPoint = value

    def _getGraftPoint(self):
        """
      Property target used to get the graft point.
      """
        return self._graftPoint

    def _setUseRockRidge(self, value):
        """
      Property target used to set the use RockRidge flag.
      No validations, but we normalize the value to ``True`` or ``False``.
      """
        if value:
            self._useRockRidge = True
        else:
            self._useRockRidge = False

    def _getUseRockRidge(self):
        """
      Property target used to get the use RockRidge flag.
      """
        return self._useRockRidge

    def _setApplicationId(self, value):
        """
      Property target used to set the application id.
      The value must be a non-empty string if it is not ``None``.
      Raises:
         ValueError: If the value is an empty string
      """
        if value is not None:
            if len(value) < 1:
                raise ValueError("The application id must be a non-empty string.")
        self._applicationId = value

    def _getApplicationId(self):
        """
      Property target used to get the application id.
      """
        return self._applicationId

    def _setBiblioFile(self, value):
        """
      Property target used to set the biblio file.
      The value must be a non-empty string if it is not ``None``.
      Raises:
         ValueError: If the value is an empty string
      """
        if value is not None:
            if len(value) < 1:
                raise ValueError("The biblio file must be a non-empty string.")
        self._biblioFile = value

    def _getBiblioFile(self):
        """
      Property target used to get the biblio file.
      """
        return self._biblioFile

    def _setPublisherId(self, value):
        """
      Property target used to set the publisher id.
      The value must be a non-empty string if it is not ``None``.
      Raises:
         ValueError: If the value is an empty string
      """
        if value is not None:
            if len(value) < 1:
                raise ValueError("The publisher id must be a non-empty string.")
        self._publisherId = value

    def _getPublisherId(self):
        """
      Property target used to get the publisher id.
      """
        return self._publisherId

    def _setPreparerId(self, value):
        """
      Property target used to set the preparer id.
      The value must be a non-empty string if it is not ``None``.
      Raises:
         ValueError: If the value is an empty string
      """
        if value is not None:
            if len(value) < 1:
                raise ValueError("The preparer id must be a non-empty string.")
        self._preparerId = value

    def _getPreparerId(self):
        """
      Property target used to get the preparer id.
      """
        return self._preparerId

    def _setVolumeId(self, value):
        """
      Property target used to set the volume id.
      The value must be a non-empty string if it is not ``None``.
      Raises:
         ValueError: If the value is an empty string
      """
        if value is not None:
            if len(value) < 1:
                raise ValueError("The volume id must be a non-empty string.")
        self._volumeId = value

    def _getVolumeId(self):
        """
      Property target used to get the volume id.
      """
        return self._volumeId

    device = property(_getDevice, _setDevice, None, "Device that image will be written to (device path or SCSI id).")
    boundaries = property(_getBoundaries, _setBoundaries, None, "Session boundaries as required by ``mkisofs``.")
    graftPoint = property(_getGraftPoint, _setGraftPoint, None, "Default image-wide graft point (see :any:`addEntry` for details).")
    useRockRidge = property(_getUseRockRidge, _setUseRockRidge, None, "Indicates whether to use RockRidge (default is ``True``).")
    applicationId = property(
        _getApplicationId, _setApplicationId, None, "Optionally specifies the ISO header application id value."
    )
    biblioFile = property(_getBiblioFile, _setBiblioFile, None, "Optionally specifies the ISO bibliographic file name.")
    publisherId = property(_getPublisherId, _setPublisherId, None, "Optionally specifies the ISO header publisher id value.")
    preparerId = property(_getPreparerId, _setPreparerId, None, "Optionally specifies the ISO header preparer id value.")
    volumeId = property(_getVolumeId, _setVolumeId, None, "Optionally specifies the ISO header volume id value.")

    #########################
    # General public methods
    #########################

    def addEntry(self, path, graftPoint=None, override=False, contentsOnly=False):
        """
      Adds an individual file or directory into the ISO image.

      The path must exist and must be a file or a directory.  By default, the
      entry will be placed into the image at the root directory, but this
      behavior can be overridden using the ``graftPoint`` parameter or instance
      variable.

      You can use the ``contentsOnly`` behavior to revert to the "original"
      ``mkisofs`` behavior for adding directories, which is to add only the
      items within the directory, and not the directory itself.

      *Note:* Things get *odd* if you try to add a directory to an image that
      will be written to a multisession disc, and the same directory already
      exists in an earlier session on that disc.  Not all of the data gets
      written.  You really wouldn't want to do this anyway, I guess.

      *Note:* An exception will be thrown if the path has already been added to
      the image, unless the ``override`` parameter is set to ``True``.

      *Note:* The method ``graftPoints`` parameter overrides the object-wide
      instance variable.  If neither the method parameter or object-wide value
      is set, the path will be written at the image root.  The graft point
      behavior is determined by the value which is in effect I{at the time this
      method is called}, so you *must* set the object-wide value before
      calling this method for the first time, or your image may not be
      consistent.

      *Note:* You *cannot* use the local ``graftPoint`` parameter to "turn off"
      an object-wide instance variable by setting it to ``None``.  Python's
      default argument functionality buys us a lot, but it can't make this
      method psychic. :)

      Args:
         path (String representing a path on disk): File or directory to be added to the image
         graftPoint (String representing a graft point path, as described above): Graft point to be used when adding this entry
         override (Boolean true/false): Override an existing entry with the same path
         contentsOnly (Boolean true/false): Add directory contents only (standard ``mkisofs`` behavior)
      Raises:
         ValueError: If path is not a file or directory, or does not exist
         ValueError: If the path has already been added, and override is not set
         ValueError: If a path cannot be encoded properly
      """
        path = encodePath(path)
        if not override:
            if path in list(self.entries.keys()):
                raise ValueError("Path has already been added to the image.")
        if os.path.islink(path):
            raise ValueError("Path must not be a link.")
        if os.path.isdir(path):
            if graftPoint is not None:
                if contentsOnly:
                    self.entries[path] = graftPoint
                else:
                    self.entries[path] = os.path.join(graftPoint, os.path.basename(path))
            elif self.graftPoint is not None:
                if contentsOnly:
                    self.entries[path] = self.graftPoint
                else:
                    self.entries[path] = os.path.join(self.graftPoint, os.path.basename(path))
            else:
                if contentsOnly:
                    self.entries[path] = None
                else:
                    self.entries[path] = os.path.basename(path)
        elif os.path.isfile(path):
            if graftPoint is not None:
                self.entries[path] = graftPoint
            elif self.graftPoint is not None:
                self.entries[path] = self.graftPoint
            else:
                self.entries[path] = None
        else:
            raise ValueError("Path must be a file or a directory.")

    def getEstimatedSize(self):
        """
      Returns the estimated size (in bytes) of the ISO image.

      This is implemented via the ``-print-size`` option to ``mkisofs``, so it
      might take a bit of time to execute.  However, the result is as accurate
      as we can get, since it takes into account all of the ISO overhead, the
      true cost of directories in the structure, etc, etc.

      Returns:
          Estimated size of the image, in bytes

      Raises:
         IOError: If there is a problem calling ``mkisofs``
         ValueError: If there are no filesystem entries in the image
      """
        if len(list(self.entries.keys())) == 0:
            raise ValueError("Image does not contain any entries.")
        return self._getEstimatedSize(self.entries)

    def _getEstimatedSize(self, entries):
        """
      Returns the estimated size (in bytes) for the passed-in entries dictionary.
      Returns:
          Estimated size of the image, in bytes
      Raises:
         IOError: If there is a problem calling ``mkisofs``
      """
        args = self._buildSizeArgs(entries)
        command = resolveCommand(MKISOFS_COMMAND)
        (result, output) = executeCommand(command, args, returnOutput=True, ignoreStderr=True)
        if result != 0:
            raise IOError("Error (%d) executing mkisofs command to estimate size." % result)
        if len(output) != 1:
            raise IOError("Unable to parse mkisofs output.")
        try:
            sectors = float(output[0])
            size = convertSize(sectors, UNIT_SECTORS, UNIT_BYTES)
            return size
        except:
            raise IOError("Unable to parse mkisofs output.")

    def writeImage(self, imagePath):
        """
      Writes this image to disk using the image path.

      Args:
         imagePath (String representing a path on disk): Path to write image out as
      Raises:
         IOError: If there is an error writing the image to disk
         ValueError: If there are no filesystem entries in the image
         ValueError: If a path cannot be encoded properly
      """
        imagePath = encodePath(imagePath)
        if len(list(self.entries.keys())) == 0:
            raise ValueError("Image does not contain any entries.")
        args = self._buildWriteArgs(self.entries, imagePath)
        command = resolveCommand(MKISOFS_COMMAND)
        (result, output) = executeCommand(command, args, returnOutput=False)
        if result != 0:
            raise IOError("Error (%d) executing mkisofs command to build image." % result)

    #########################################
    # Methods used to build mkisofs commands
    #########################################

    @staticmethod
    def _buildDirEntries(entries):
        """
      Uses an entries dictionary to build a list of directory locations for use
      by ``mkisofs``.

      We build a list of entries that can be passed to ``mkisofs``.  Each entry is
      either raw (if no graft point was configured) or in graft-point form as
      described above (if a graft point was configured).  The dictionary keys
      are the path names, and the values are the graft points, if any.

      Args:
         entries: Dictionary of image entries (i.e. self.entries)

      Returns:
          List of directory locations for use by ``mkisofs``
      """
        dirEntries = []
        for key in list(entries.keys()):
            if entries[key] is None:
                dirEntries.append(key)
            else:
                dirEntries.append("%s/=%s" % (entries[key].strip("/"), key))
        return dirEntries

    def _buildGeneralArgs(self):
        """
      Builds a list of general arguments to be passed to a ``mkisofs`` command.

      The various instance variables (``applicationId``, etc.) are filled into
      the list of arguments if they are set.
      By default, we will build a RockRidge disc.  If you decide to change
      this, think hard about whether you know what you're doing.  This option
      is not well-tested.

      Returns:
          List suitable for passing to :any:`util.executeCommand` as ``args``
      """
        args = []
        if self.applicationId is not None:
            args.append("-A")
            args.append(self.applicationId)
        if self.biblioFile is not None:
            args.append("-biblio")
            args.append(self.biblioFile)
        if self.publisherId is not None:
            args.append("-publisher")
            args.append(self.publisherId)
        if self.preparerId is not None:
            args.append("-p")
            args.append(self.preparerId)
        if self.volumeId is not None:
            args.append("-V")
            args.append(self.volumeId)
        return args

    def _buildSizeArgs(self, entries):
        """
      Builds a list of arguments to be passed to a ``mkisofs`` command.

      The various instance variables (``applicationId``, etc.) are filled into
      the list of arguments if they are set.  The command will be built to just
      return size output (a simple count of sectors via the ``-print-size`` option),
      rather than an image file on disk.

      By default, we will build a RockRidge disc.  If you decide to change
      this, think hard about whether you know what you're doing.  This option
      is not well-tested.

      Args:
         entries: Dictionary of image entries (i.e. self.entries)

      Returns:
          List suitable for passing to :any:`util.executeCommand` as ``args``
      """
        args = self._buildGeneralArgs()
        args.append("-print-size")
        args.append("-graft-points")
        if self.useRockRidge:
            args.append("-r")
        if self.device is not None and self.boundaries is not None:
            args.append("-C")
            args.append("%d,%d" % (self.boundaries[0], self.boundaries[1]))
            args.append("-M")
            args.append(self.device)
        args.extend(self._buildDirEntries(entries))
        return args

    def _buildWriteArgs(self, entries, imagePath):
        """
      Builds a list of arguments to be passed to a ``mkisofs`` command.

      The various instance variables (``applicationId``, etc.) are filled into
      the list of arguments if they are set.  The command will be built to write
      an image to disk.

      By default, we will build a RockRidge disc.  If you decide to change
      this, think hard about whether you know what you're doing.  This option
      is not well-tested.

      Args:
         entries: Dictionary of image entries (i.e. self.entries)

         imagePath (String representing a path on disk): Path to write image out as
      Returns:
          List suitable for passing to :any:`util.executeCommand` as ``args``
      """
        args = self._buildGeneralArgs()
        args.append("-graft-points")
        if self.useRockRidge:
            args.append("-r")
        args.append("-o")
        args.append(imagePath)
        if self.device is not None and self.boundaries is not None:
            args.append("-C")
            args.append("%d,%d" % (self.boundaries[0], self.boundaries[1]))
            args.append("-M")
            args.append(self.device)
        args.extend(self._buildDirEntries(entries))
        return args



# -*- coding: iso-8859-1 -*-
# vim: set ft=python ts=4 sw=4 expandtab:
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
#              C E D A R
#          S O L U T I O N S       "Software done right."
#           S O F T W A R E
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
# Copyright (c) 2004-2007,2010,2015 Kenneth J. Pronovici.
# All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License,
# Version 2, as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# Copies of the GNU General Public License are available from
# the Free Software Foundation website, http://www.gnu.org/.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
# Author   : Kenneth J. Pronovici <pronovic@ieee.org>
# Language : Python 3 (>= 3.7)
# Project  : Cedar Backup, release 3
# Purpose  : Implements the standard 'validate' action.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

########################################################################
# Module documentation
########################################################################

"""
Implements the standard 'validate' action.
:author: Kenneth J. Pronovici <pronovic@ieee.org>
"""


########################################################################
# Imported modules
########################################################################

import logging
import os

from CedarBackup3.actions.util import createWriter
from CedarBackup3.util import getFunctionReference, getUidGid

########################################################################
# Module-wide constants and variables
########################################################################

logger = logging.getLogger("CedarBackup3.log.actions.validate")


########################################################################
# Public functions
########################################################################

#############################
# executeValidate() function
#############################

# pylint: disable=W0613
def executeValidate(configPath, options, config):
    """
   Executes the validate action.

   This action validates each of the individual sections in the config file.
   This is a "runtime" validation.  The config file itself is already valid in
   a structural sense, so what we check here that is that we can actually use
   the configuration without any problems.

   There's a separate validation function for each of the configuration
   sections.  Each validation function returns a true/false indication for
   whether configuration was valid, and then logs any configuration problems it
   finds.  This way, one pass over configuration indicates most or all of the
   obvious problems, rather than finding just one problem at a time.

   Any reported problems will be logged at the ERROR level normally, or at the
   INFO level if the quiet flag is enabled.

   Args:
      configPath (String representing a path on disk): Path to configuration file on disk
      options (Options object): Program command-line options
      config (Config object): Program configuration
   Raises:
      ValueError: If some configuration value is invalid
   """
    logger.debug("Executing the 'validate' action.")
    if options.quiet:
        logfunc = logger.info  # info so it goes to the log
    else:
        logfunc = logger.error  # error so it goes to the screen
    valid = True
    valid &= _validateReference(config, logfunc)
    valid &= _validateOptions(config, logfunc)
    valid &= _validateCollect(config, logfunc)
    valid &= _validateStage(config, logfunc)
    valid &= _validateStore(config, logfunc)
    valid &= _validatePurge(config, logfunc)
    valid &= _validateExtensions(config, logfunc)
    if valid:
        logfunc("Configuration is valid.")
    else:
        logfunc("Configuration is not valid.")


########################################################################
# Private utility functions
########################################################################

#######################
# _checkDir() function
#######################


def _checkDir(path, writable, logfunc, prefix):
    """
   Checks that the indicated directory is OK.

   The path must exist, must be a directory, must be readable and executable,
   and must optionally be writable.

   Args:
      path: Path to check
      writable: Check that path is writable
      logfunc: Function to use for logging errors
      prefix: Prefix to use on logged errors

   Returns:
       True if the directory is OK, False otherwise
   """
    if not os.path.exists(path):
        logfunc("%s [%s] does not exist." % (prefix, path))
        return False
    if not os.path.isdir(path):
        logfunc("%s [%s] is not a directory." % (prefix, path))
        return False
    if not os.access(path, os.R_OK):
        logfunc("%s [%s] is not readable." % (prefix, path))
        return False
    if not os.access(path, os.X_OK):
        logfunc("%s [%s] is not executable." % (prefix, path))
        return False
    if writable and not os.access(path, os.W_OK):
        logfunc("%s [%s] is not writable." % (prefix, path))
        return False
    return True


################################
# _validateReference() function
################################


def _validateReference(config, logfunc):
    """
   Execute runtime validations on reference configuration.

   We only validate that reference configuration exists at all.

   Args:
      config: Program configuration
      logfunc: Function to use for logging errors

   Returns:
       True if configuration is valid, false otherwise
   """
    valid = True
    if config.reference is None:
        logfunc("Required reference configuration does not exist.")
        valid = False
    return valid


##############################
# _validateOptions() function
##############################


def _validateOptions(config, logfunc):
    """
   Execute runtime validations on options configuration.

   The following validations are enforced:

      - The options section must exist
      - The working directory must exist and must be writable
      - The backup user and backup group must exist

   Args:
      config: Program configuration
      logfunc: Function to use for logging errors

   Returns:
       True if configuration is valid, false otherwise
   """
    valid = True
    if config.options is None:
        logfunc("Required options configuration does not exist.")
        valid = False
    else:
        valid &= _checkDir(config.options.workingDir, True, logfunc, "Working directory")
        try:
            getUidGid(config.options.backupUser, config.options.backupGroup)
        except ValueError:
            logfunc("Backup user:group [%s:%s] invalid." % (config.options.backupUser, config.options.backupGroup))
            valid = False
    return valid


##############################
# _validateCollect() function
##############################


def _validateCollect(config, logfunc):
    """
   Execute runtime validations on collect configuration.

   The following validations are enforced:

      - The target directory must exist and must be writable
      - Each of the individual collect directories must exist and must be readable

   Args:
      config: Program configuration
      logfunc: Function to use for logging errors

   Returns:
       True if configuration is valid, false otherwise
   """
    valid = True
    if config.collect is not None:
        valid &= _checkDir(config.collect.targetDir, True, logfunc, "Collect target directory")
        if config.collect.collectDirs is not None:
            for collectDir in config.collect.collectDirs:
                valid &= _checkDir(collectDir.absolutePath, False, logfunc, "Collect directory")
    return valid


############################
# _validateStage() function
############################


def _validateStage(config, logfunc):
    """
   Execute runtime validations on stage configuration.

   The following validations are enforced:

      - The target directory must exist and must be writable
      - Each local peer's collect directory must exist and must be readable

   *Note:* We currently do not validate anything having to do with remote peers,
   since we don't have a straightforward way of doing it.  It would require
   adding an rsh command rather than just an rcp command to configuration, and
   that just doesn't seem worth it right now.

   Args:
      config: Program configuration
      logfunc: Function to use for logging errors

   Returns:
       True if configuration is valid, False otherwise
   """
    valid = True
    if config.stage is not None:
        valid &= _checkDir(config.stage.targetDir, True, logfunc, "Stage target dir ")
        if config.stage.localPeers is not None:
            for peer in config.stage.localPeers:
                valid &= _checkDir(peer.collectDir, False, logfunc, "Local peer collect dir ")
    return valid


############################
# _validateStore() function
############################


def _validateStore(config, logfunc):
    """
   Execute runtime validations on store configuration.

   The following validations are enforced:

      - The source directory must exist and must be readable
      - The backup device (path and SCSI device) must be valid

   Args:
      config: Program configuration
      logfunc: Function to use for logging errors

   Returns:
       True if configuration is valid, False otherwise
   """
    valid = True
    if config.store is not None:
        valid &= _checkDir(config.store.sourceDir, False, logfunc, "Store source directory")
        try:
            createWriter(config)
        except ValueError:
            logfunc("Backup device [%s] [%s] is not valid." % (config.store.devicePath, config.store.deviceScsiId))
            valid = False
    return valid


############################
# _validatePurge() function
############################


def _validatePurge(config, logfunc):
    """
   Execute runtime validations on purge configuration.

   The following validations are enforced:

      - Each purge directory must exist and must be writable

   Args:
      config: Program configuration
      logfunc: Function to use for logging errors

   Returns:
       True if configuration is valid, False otherwise
   """
    valid = True
    if config.purge is not None:
        if config.purge.purgeDirs is not None:
            for purgeDir in config.purge.purgeDirs:
                valid &= _checkDir(purgeDir.absolutePath, True, logfunc, "Purge directory")
    return valid


#################################
# _validateExtensions() function
#################################


def _validateExtensions(config, logfunc):
    """
   Execute runtime validations on extensions configuration.

   The following validations are enforced:

      - Each indicated extension function must exist.

   Args:
      config: Program configuration
      logfunc: Function to use for logging errors

   Returns:
       True if configuration is valid, False otherwise
   """
    valid = True
    if config.extensions is not None:
        if config.extensions.actions is not None:
            for action in config.extensions.actions:
                try:
                    getFunctionReference(action.module, action.function)
                except ImportError:
                    logfunc("Unable to find function [%s.%s]." % (action.module, action.function))
                    valid = False
                except ValueError:
                    logfunc("Function [%s.%s] is not callable." % (action.module, action.function))
                    valid = False
    return valid
